/*
 * JNpp - Java bindings for NPP, to be used with JCuda
 *
 * Copyright (c) 2010-2012 Marco Hutter - http://www.jcuda.org
 *
 * Permission is hereby granted, free of charge, to any person
 * obtaining a copy of this software and associated documentation
 * files (the "Software"), to deal in the Software without
 * restriction, including without limitation the rights to use,
 * copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following
 * conditions:
 *
 * The above copyright notice and this permission notice shall be
 * included in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
 * EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES
 * OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
 * NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
 * HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY,
 * WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
 * FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR
 * OTHER DEALINGS IN THE SOFTWARE.
 */


#include "JNppi.hpp"
#include "JNppi_common.hpp"
#include <iostream>
#include <string>
#include <map>


jfieldID cuComplex_x; // float
jfieldID cuComplex_y; // float

jfieldID cuDoubleComplex_x; // double
jfieldID cuDoubleComplex_y; // double

jfieldID NppiPoint_x; // int
jfieldID NppiPoint_y; // int

jfieldID NppiSize_width; // int
jfieldID NppiSize_height; // int

jfieldID NppiRect_x; // int
jfieldID NppiRect_y; // int
jfieldID NppiRect_width; // int
jfieldID NppiRect_height; // int


jfieldID Npp16sc_re; // short
jfieldID Npp16sc_im; // short

jfieldID Npp32sc_re; // int
jfieldID Npp32sc_im; // int

jfieldID Npp64sc_re; // long
jfieldID Npp64sc_im; // long

jfieldID Npp32fc_re; // float
jfieldID Npp32fc_im; // float

jfieldID Npp64fc_re; // double
jfieldID Npp64fc_im; // double


// Class and method ID for TypedPointer and its constructor
jclass TypedPointer_Class;
jmethodID TypedPointer_Constructor;


/**
 * Creates a global reference to the class with the given name and
 * stores it in the given jclass argument, and stores the no-args
 * constructor ID for this class in the given jmethodID.
 * Returns whether this initialization succeeded.
 */
bool init(JNIEnv *env, const char *className, jclass &globalCls, jmethodID &constructor)
{
    jclass cls = NULL;
    if (!init(env, cls, className)) return false;
    if (!init(env, cls, constructor, "<init>", "()V")) return false;

    globalCls = (jclass)env->NewGlobalRef(cls);
    if (globalCls == NULL)
    {
        Logger::log(LOG_ERROR, "Failed to create reference to class %s\n", className);
        return false;
    }
    return true;
}

/**
 * Called when the library is loaded. Will initialize all
 * required field and method IDs
 */
JNIEXPORT jint JNICALL JNI_OnLoad(JavaVM *jvm, void *reserved)
{
    JNIEnv *env = NULL;
    if (jvm->GetEnv((void **)&env, JNI_VERSION_1_4))
    {
        return JNI_ERR;
    }

    Logger::log(LOG_TRACE, "Initializing JNppi\n");

    jclass cls = NULL;

    // Initialize the JNIUtils and PointerUtils
    if (initJNIUtils(env) == JNI_ERR) return JNI_ERR;
    if (initPointerUtils(env) == JNI_ERR) return JNI_ERR;

    // Obtain the constructor for the TypedPointer class
    if (!init(env, "jcuda/jnpp/TypedPointer", TypedPointer_Class, TypedPointer_Constructor)) return JNI_ERR;

    // Obtain the fieldIDs for cuComplex#x and cuComplex#y
    if (!init(env, cls, "jcuda/cuComplex")) return JNI_ERR;
    if (!init(env, cls, cuComplex_x, "x", "F")) return JNI_ERR;
    if (!init(env, cls, cuComplex_y, "y", "F")) return JNI_ERR;

    // Obtain the fieldIDs for cuDoubleComplex#x and cuDoubleComplex#y
    if (!init(env, cls, "jcuda/cuDoubleComplex")) return JNI_ERR;
    if (!init(env, cls, cuDoubleComplex_x, "x", "D")) return JNI_ERR;
    if (!init(env, cls, cuDoubleComplex_y, "y", "D")) return JNI_ERR;

    // Obtain the fieldIDs for NppiPoint
    if (!init(env, cls, "jcuda/jnpp/NppiPoint")) return JNI_ERR;
    if (!init(env, cls, NppiPoint_x, "x", "I")) return JNI_ERR;
    if (!init(env, cls, NppiPoint_y, "y", "I")) return JNI_ERR;

    // Obtain the fieldIDs for NppiSize
    if (!init(env, cls, "jcuda/jnpp/NppiSize")) return JNI_ERR;
    if (!init(env, cls, NppiSize_width,  "width",  "I")) return JNI_ERR;
    if (!init(env, cls, NppiSize_height, "height", "I")) return JNI_ERR;

    // Obtain the fieldIDs for NppiRect
    if (!init(env, cls, "jcuda/jnpp/NppiRect")) return JNI_ERR;
    if (!init(env, cls, NppiRect_x,      "x",      "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_y,      "y",      "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_width,  "width",  "I")) return JNI_ERR;
    if (!init(env, cls, NppiRect_height, "height", "I")) return JNI_ERR;

    // Obtain the fieldIDs for Npp16sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp16sc")) return JNI_ERR;
    if (!init(env, cls, Npp16sc_re, "re", "S")) return JNI_ERR;
    if (!init(env, cls, Npp16sc_im, "im", "S")) return JNI_ERR;

    // Obtain the fieldIDs for Npp32sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp32sc")) return JNI_ERR;
    if (!init(env, cls, Npp32sc_re, "re", "I")) return JNI_ERR;
    if (!init(env, cls, Npp32sc_im, "im", "I")) return JNI_ERR;

    // Obtain the fieldIDs for Npp64sc
    if (!init(env, cls, "jcuda/jnpp/types/Npp64sc")) return JNI_ERR;
    if (!init(env, cls, Npp64sc_re, "re", "J")) return JNI_ERR;
    if (!init(env, cls, Npp64sc_im, "im", "J")) return JNI_ERR;

    // Obtain the fieldIDs for Npp32fc
    if (!init(env, cls, "jcuda/jnpp/types/Npp32fc")) return JNI_ERR;
    if (!init(env, cls, Npp32fc_re, "re", "F")) return JNI_ERR;
    if (!init(env, cls, Npp32fc_im, "im", "F")) return JNI_ERR;

    // Obtain the fieldIDs for Npp64fc
    if (!init(env, cls, "jcuda/jnpp/types/Npp64fc")) return JNI_ERR;
    if (!init(env, cls, Npp64fc_re, "re", "D")) return JNI_ERR;
    if (!init(env, cls, Npp64fc_im, "im", "D")) return JNI_ERR;

    return JNI_VERSION_1_4;

}


/**
 * Create a new TypedPointer object with the given native pointer value
 */
jobject createPointer(JNIEnv *env, jlong pointerValue)
{
    jobject result = env->NewObject(TypedPointer_Class, TypedPointer_Constructor);
    if (env->ExceptionCheck())
    {
        return NULL;
    }
    setPointer(env, result, pointerValue);
    return result;
}


/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiPoint &output)
{
    output.x = (int)env->GetIntField(input, NppiPoint_x);
    output.y = (int)env->GetIntField(input, NppiPoint_y);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiSize &output)
{
    output.width  = (int)env->GetIntField(input, NppiSize_width);
    output.height = (int)env->GetIntField(input, NppiSize_height);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, NppiRect &output)
{
    output.x      = (int)env->GetIntField(input, NppiRect_x);
    output.y      = (int)env->GetIntField(input, NppiRect_y);
    output.width  = (int)env->GetIntField(input, NppiRect_width);
    output.height = (int)env->GetIntField(input, NppiRect_height);
    return true;
}


//=== Complex number struct init =============================================

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp16sc &output)
{
    output.re = (Npp16s)env->GetShortField(input, Npp16sc_re);
    output.im = (Npp16s)env->GetShortField(input, Npp16sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp32sc &output)
{
    output.re = (Npp32s)env->GetIntField(input, Npp32sc_re);
    output.im = (Npp32s)env->GetIntField(input, Npp32sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp64sc &output)
{
    output.re = (Npp64s)env->GetLongField(input, Npp64sc_re);
    output.im = (Npp64s)env->GetLongField(input, Npp64sc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp32fc &output)
{
    output.re = (Npp32f)env->GetFloatField(input, Npp32fc_re);
    output.im = (Npp32f)env->GetFloatField(input, Npp32fc_im);
    return true;
}

/**
 * Initializes the given native output object with the values from the
 * given java input object.
 */
bool initNative(JNIEnv *env, jobject input, Npp64fc &output)
{
    output.re = (Npp64f)env->GetDoubleField(input, Npp64fc_re);
    output.im = (Npp64f)env->GetDoubleField(input, Npp64fc_im);
    return true;
}



//=== Complex number struct release ==========================================

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp16sc &output, jobject input)
{
    env->SetShortField(input, Npp16sc_re, (jshort)output.re);
    env->SetShortField(input, Npp16sc_im, (jshort)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp32sc &output, jobject input)
{
    env->SetIntField(input, Npp32sc_re, (jint)output.re);
    env->SetIntField(input, Npp32sc_im, (jint)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp64sc &output, jobject input)
{
    env->SetLongField(input, Npp64sc_re, (jlong)output.re);
    env->SetLongField(input, Npp64sc_im, (jlong)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp32fc &output, jobject input)
{
    env->SetFloatField(input, Npp32fc_re, (jfloat)output.re);
    env->SetFloatField(input, Npp32fc_im, (jfloat)output.im);
    return true;
}

/**
 * Write the values from the given native output object back to
 * the java input object
 */
bool releaseNative(JNIEnv *env, Npp64fc &output, jobject input)
{
    env->SetDoubleField(input, Npp64fc_re, (jdouble)output.re);
    env->SetDoubleField(input, Npp64fc_im, (jdouble)output.im);
    return true;
}





//=== 2D prmitive arrays =====================================================


// TODO: Many of these methods could be replaced by more generic
// ones that should be defined in CommonJNI

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, double output[4][2])
{
    int outerSize = 4;
    int innerSize = 2;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (double)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, double output[4][2], jobjectArray input)
{
    int outerSize = 4;
    int innerSize = 2;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jdouble)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}





/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, double output[2][3])
{
    int outerSize = 2;
    int innerSize = 3;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (double)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, double output[2][3], jobjectArray input)
{
    int outerSize = 2;
    int innerSize = 3;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jdoubleArray inputElement = (jdoubleArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jdouble *arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jdouble)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32f output[3][4])
{
    int outerSize = 3;
    int innerSize = 4;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jfloatArray inputElement = (jfloatArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jfloat *arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            output[i][j] = (Npp32f)arrayContents[j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, JNI_ABORT);
    }
    return true;
}

/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32f output[3][4], jobjectArray input)
{
    int outerSize = 3;
    int innerSize = 4;
    jsize size = env->GetArrayLength(input);
    if (size < outerSize)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<outerSize; i++)
    {
        jfloatArray inputElement = (jfloatArray)env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException", "Array contains 'null' elements");
            return false;
        }
        jsize elementSize = env->GetArrayLength(inputElement);
        if (elementSize < innerSize)
        {
            ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException", "Array parameter has insufficient size");
            return false;
        }
        jfloat *arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(inputElement, NULL);
        for (int j=0; j<innerSize; j++)
        {
            arrayContents[j] = (jfloat)output[i][j];
        }
        env->ReleasePrimitiveArrayCritical(inputElement, arrayContents, 0);
    }
    return true;
}






//=== Primitive arrays =======================================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jbyteArray input, Npp8s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp8s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp8s output[], jbyteArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jbyte)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jbyteArray input, Npp8u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp8u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp8u output[], jbyteArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jbyte* arrayContents = (jbyte*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jbyte)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jshortArray input, Npp16s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp16s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16s output[], jshortArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jshort)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jshortArray input, Npp16u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp16u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16u output[], jshortArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jshort* arrayContents = (jshort*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jshort)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jintArray input, Npp32s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32s output[], jintArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jint)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jintArray input, Npp32u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32u output[], jintArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jint* arrayContents = (jint*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jint)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jlongArray input, Npp64s output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64s)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64s output[], jlongArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jlong)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jlongArray input, Npp64u output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64u)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64u output[], jlongArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jlong* arrayContents = (jlong*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jlong)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jfloatArray input, Npp32f output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jfloat* arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp32f)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32f output[], jfloatArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jfloat* arrayContents = (jfloat*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jfloat)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jdoubleArray input, Npp64f output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jdouble* arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        output[i] = (Npp64f)arrayContents[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, JNI_ABORT);
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64f output[], jdoubleArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    jdouble* arrayContents = (jdouble*)env->GetPrimitiveArrayCritical(input, NULL);
    if (arrayContents == NULL)
    {
        return false;
    }
    for (int i=0; i<size; i++)
    {
        arrayContents[i] = (jdouble)output[i];
    }
    env->ReleasePrimitiveArrayCritical(input, arrayContents, 0);
    return true;
}





//=== Complex number arrays ==================================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16sc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp16sc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32sc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32sc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32fc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp32fc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}



/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp64fc output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!initNative(env, inputElement, output[i]))
        {
            return false;
        }
    }
    return true;
}


/**
 * Release the given native output array, writing its contents back to
 * the given java input array
 */
bool releaseNative(JNIEnv *env, Npp64fc output[], jobjectArray input, int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        if (!releaseNative(env, output[i], inputElement))
        {
            return false;
        }
    }
    return true;
}





//=== Primitive pointer arrays ===============================================

/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp8u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp8u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp8u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp8u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp16s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp16s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp16s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp16u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp16u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp16u* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp16u*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp32s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp32s* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp32s*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, Npp32f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (Npp32f*)getPointer(env, inputElement);
    }
    return true;
}




/**
 * Initialize the given native output array by filling it with the data from
 * the given java input array
 */
bool initNative(JNIEnv *env, jobjectArray input, const Npp32f* output[], int size)
{
    jsize arraySize = env->GetArrayLength(input);
    if (arraySize < size)
    {
        ThrowByName(env, "java/lang/ArrayIndexOutOfBoundsException",
            "Array parameter has insufficient size");
        return false;
    }
    for (int i=0; i<size; i++)
    {
        jobject inputElement = env->GetObjectArrayElement(input, i);
        if (inputElement == NULL)
        {
            ThrowByName(env, "java/lang/NullPointerException",
                "Input array contains 'null' elements");
            return false;
        }
        output[i] = (const Npp32f*)getPointer(env, inputElement);
    }
    return true;
}























/*
 * Class:     jcuda_jnpp_JNppi
 * Method:    setLogLevelNative
 * Signature: (I)V
 */
JNIEXPORT void JNICALL Java_jcuda_jnpp_JNppi_setLogLevelNative
  (JNIEnv *env, jclass cla, jint logLevel)
{
    Logger::setLogLevel((LogLevel)logLevel);
}









//=== Auto-generated part: ===================================================

/**
 * <pre>
 * 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_18u_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_8u_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_8u_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp8u* jniResult_native = nppiMalloc_8u_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116u_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16u_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16u_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16u* jniResult_native = nppiMalloc_16u_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 16-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116s_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16s_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16s_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16s* jniResult_native = nppiMalloc_16s_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 1 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 16-bit signed complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_116sc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_16sc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_16sc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp16sc* jniResult_native = nppiMalloc_16sc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit signed image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132s_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32s_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32s_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32s* jniResult_native = nppiMalloc_32s_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit integer complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132sc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32sc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32sc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32sc* jniResult_native = nppiMalloc_32sc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit floating point image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132f_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32f_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32f_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32f* jniResult_native = nppiMalloc_32f_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C1Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C1");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C1(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C1(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 2 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C2Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C2");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C2(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C2(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 3 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C3Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C3");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C3(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C3(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

/**
 * <pre>
 * 4 channel 32-bit float complex image memory allocator.
 * @param nWidthPixels Image width.
 * @param nHeightPixels Image height.
 * @param pStepBytes \ref line_step.
 * @return
 *  Pointer to new image data.
 * </pre>
 */
JNIEXPORT jobject JNICALL Java_jcuda_jnpp_JNppi_nppiMalloc_132fc_1C4Native(JNIEnv *env, jclass cls, jint nWidthPixels, jint nHeightPixels, jintArray pStepBytes)
{
    // Null-checks for non-primitive arguments
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (pStepBytes == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStepBytes' is null for nppiMalloc_32fc_C4");
        return NULL;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMalloc_32fc_C4(nWidthPixels=%d, nHeightPixels=%d, pStepBytes=%p)\n",
        nWidthPixels, nHeightPixels, pStepBytes);

    // Native variable declarations
    int nWidthPixels_native = 0;
    int nHeightPixels_native = 0;
    int pStepBytes_native = 0;

    // Obtain native variable values
    nWidthPixels_native = (int)nWidthPixels;
    nHeightPixels_native = (int)nHeightPixels;
    // pStepBytes is set here

    // Native function call
    Npp32fc* jniResult_native = nppiMalloc_32fc_C4(nWidthPixels_native, nHeightPixels_native, &pStepBytes_native);

    // Write back native variable values
    // nWidthPixels is primitive
    // nHeightPixels is primitive
    if (!set(env, pStepBytes, 0, (jint)pStepBytes_native)) return NULL;

    // Return the result
    return createPointer(env, (jlong)jniResult_native);
}

///@} Malloc
/**
 * <pre>
 * Free method for any 2D allocated memory.
 * This method should be used to free memory allocated with any of the nppiMalloc_<modifier> methods.
 * @param pData A pointer to memory allocated using nppiMalloc_<modifier>.
 * </pre>
 */
JNIEXPORT void JNICALL Java_jcuda_jnpp_JNppi_nppiFreeNative(JNIEnv *env, jclass cls, jobject pData)
{
    // Null-checks for non-primitive arguments
    if (pData == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pData' is null for nppiFree");
        return;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFree(pData=%p)\n",
        pData);

    // Native variable declarations
    void* pData_native;

    // Obtain native variable values
    PointerData *pData_pointerData = initPointerData(env, pData);
    if (pData_pointerData == NULL)
    {
        return;
    }
    pData_native = (void*)pData_pointerData->getPointer(env);

    // Native function call
    nppiFree(pData_native);

    // Write back native variable values
    if (!releasePointerData(env, pData_pointerData, 0)) return;

}

///@} Image-Memory Management
/**
 * <pre>
 * @defgroup image_data_exchange_and_initialization Data-Exchange and Initialization
 *
 * Primitives for initializtion, copying and converting image data.
 *
 * </pre>
 */
///@{
/**
 * <pre>
 * @name Image-Memory Set
 * Set methods for images of various types. Images are passed to these primitives via a pointer
 * to the image data (first pixel in the ROI) and a step-width, i.e. the number of bytes between
 * successive lines. The size of the area to be set (region-of-interest, ROI) is specified via
 * a NppiSize struct. 
 * In addition to the image data and ROI, all methods have a parameter to specify the value being
 * set. In case of single channel images this is a single value, in case of multi-channel, an 
 * array of values is passed. 
 * </pre>
 */
///@{
/**
 * <pre>
 * 8-bit image set.
 * @param nValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C1RNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C1R(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s nValue_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit two-channel image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C2RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[2];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit three-channel image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C3RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit four-channel image set.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1C4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[4];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit four-channel image set ignoring alpha channel.
 * @param aValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18s_1AC4RNative(JNIEnv *env, jclass cls, jbyteArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8s_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s aValue_native[3];
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8s_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image set.
 * @param nValue The pixel value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C1RNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C1R(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 8-bit unsigned image set. 
 * The 8-bit mask image affects setting of the respective pixels in the destination image.
 * If the mask value is zero (0) the pixel is not set, if the mask is non-zero, the corresponding
 * destination pixel is set to specified value.
 * @param nValue The pixel value to be set.
 * @param pDst Pointer \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C1MRNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C1MR(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image set.
 * @param aValues Four-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4RNative(JNIEnv *env, jclass cls, jbyteArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValues_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 8-bit unsigned image set.
 * @param aValues Four-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4MRNative(JNIEnv *env, jclass cls, jbyteArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u aValues_native[4];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1AC4RNative(JNIEnv *env, jclass cls, jbyteArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_AC4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u aValues_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_AC4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 8-bit unsigned image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1AC4MRNative(JNIEnv *env, jclass cls, jbyteArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_AC4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u aValues_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_AC4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image set affecting only single channel.
 * For RGBA images, this method allows setting of a single of the four (RGBA) values 
 * without changing the contents of the other three channels. The channel is selected
 * via the pDst pointer. The pointer needs to point to the actual first value to be set,
 * e.g. in order to set the R-channel (first channel), one would pass pDst unmodified, since
 * its value actually points to the r channel. If one wanted to modify the B channel (second
 * channel), one would pass pDst + 2 to the function.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_18u_1C4CRNative(JNIEnv *env, jclass cls, jbyte nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_8u_C4CR(nValue=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nValue_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp8u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_8u_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C1RNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 16-bit unsigned image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C1MRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit unsigned image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C2RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C2R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValues_native[2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C2R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValues_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit unsigned image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4MRNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u aValues_native[4];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1AC4RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_AC4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u aValues_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_AC4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit unsigned image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1AC4MRNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_AC4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u aValues_native[3];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_AC4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set affecting only single channel.
 * For RGBA images, this method allows setting of a single of the four (RGBA) values 
 * without changing the contents of the other three channels. The channel is selected
 * via the pDst pointer. The pointer needs to point to the actual first value to be set,
 * e.g. in order to set the R-channel (first channel), one would pass pDst unmodified, since
 * its value actually points to the r channel. If one wanted to modify the B channel (second
 * channel), one would pass pDst + 2 to the function.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116u_1C4CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16u_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nValue_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16u)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16u_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C1RNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 16-bit image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C1MRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 16-bit image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C2RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C2R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValues_native[2];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C2R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValues_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4MRNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s aValues_native[4];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1AC4RNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_AC4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s aValues_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_AC4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1AC4MRNative(JNIEnv *env, jclass cls, jshortArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_AC4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s aValues_native[3];
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_AC4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image set affecting only single channel.
 * For RGBA images, this method allows setting of a single of the four (RGBA) values 
 * without changing the contents of the other three channels. The channel is selected
 * via the pDst pointer. The pointer needs to point to the actual first value to be set,
 * e.g. in order to set the R-channel (first channel), one would pass pDst unmodified, since
 * its value actually points to the r channel. If one wanted to modify the B channel (second
 * channel), one would pass pDst + 2 to the function.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116s_1C4CRNative(JNIEnv *env, jclass cls, jshort nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16s_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s nValue_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp16s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16s_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer image set.
 * @param oValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc oValue_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer two-channel image set.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[2];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer three-channel image set.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer four-channel image set ignoring alpha.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[3];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex integer four-channel image set.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_116sc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_16sc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc aValue_native[4];
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_16sc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C1RNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C1R(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 32-bit image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C1MRNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C1MR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4RNative(JNIEnv *env, jclass cls, jintArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValues_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4MRNative(JNIEnv *env, jclass cls, jintArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s aValues_native[4];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1AC4RNative(JNIEnv *env, jclass cls, jintArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_AC4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s aValues_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_AC4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 16-bit image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1AC4MRNative(JNIEnv *env, jclass cls, jintArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_AC4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s aValues_native[3];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_AC4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit unsigned image set affecting only single channel.
 * For RGBA images, this method allows setting of a single of the four (RGBA) values 
 * without changing the contents of the other three channels. The channel is selected
 * via the pDst pointer. The pointer needs to point to the actual first value to be set,
 * e.g. in order to set the R-channel (first channel), one would pass pDst unmodified, since
 * its value actually points to the r channel. If one wanted to modify the B channel (second
 * channel), one would pass pDst + 2 to the function.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132s_1C4CRNative(JNIEnv *env, jclass cls, jint nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32s_C4CR(nValue=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nValue_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32s)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32s_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit complex integer image set.
 * @param oValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc oValue_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[2];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit complex integer image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[4];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex integer four-channel image set ignoring alpha.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132sc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32sc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc aValue_native[3];
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32sc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C1RNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C1R(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C1R(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 32-bit floating point image set.
 * @param nValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C1MRNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C1MR(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        nValue, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C1MR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4RNative(JNIEnv *env, jclass cls, jfloatArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValues_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit floating point image set.
 * @param aValues New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4MRNative(JNIEnv *env, jclass cls, jfloatArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f aValues_native[4];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1AC4RNative(JNIEnv *env, jclass cls, jfloatArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_AC4R(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValues, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f aValues_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_AC4R(aValues_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Masked 4 channel 32-bit floating point image set method, not affecting Alpha channel.
 * For RGBA images, this method allows setting of the RGB values without changing the contents
 * of the alpha-channel (fourth channel).
 * @param aValues Three-channel array containing the pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the mask image. This is a single channel 8-bit unsigned int image.
 * @param nMaskStep Number of bytes between line starts of successive lines in the mask image.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1AC4MRNative(JNIEnv *env, jclass cls, jfloatArray aValues, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (aValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValues' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiSet_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_AC4MR(aValues=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        aValues, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f aValues_native[3];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    if (!initNative(env, aValues, aValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_AC4MR(aValues_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releaseNative(env, aValues_native, aValues, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image set affecting only single channel.
 * For RGBA images, this method allows setting of a single of the four (RGBA) values 
 * without changing the contents of the other three channels. The channel is selected
 * via the pDst pointer. The pointer needs to point to the actual first value to be set,
 * e.g. in order to set the R-channel (first channel), one would pass pDst unmodified, since
 * its value actually points to the r channel. If one wanted to modify the B channel (second
 * channel), one would pass pDst + 2 to the function.
 * @param nValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132f_1C4CRNative(JNIEnv *env, jclass cls, jfloat nValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nValue is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32f_C4CR(nValue=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        nValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nValue_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nValue_native = (Npp32f)nValue;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32f_C4CR(nValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nValue is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single channel 32-bit complex image set.
 * @param oValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject oValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (oValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oValue' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C1R(oValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        oValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc oValue_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, oValue, oValue_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C1R(oValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, oValue_native, oValue)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two channel 32-bit complex image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C2RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C2R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[2];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C2R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 2)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three channel 32-bit complex image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C3RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C3R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C3R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four channel 32-bit complex image set.
 * @param aValue The pixel-value to be set.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1C4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_C4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[4];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_C4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex four-channel image set ignoring alpha.
 * @param aValue New pixel value.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSet_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobjectArray aValue, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSet_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSet_32fc_AC4R(aValue=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aValue, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc aValue_native[3];
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSet_32fc_AC4R(aValue_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Image-Memory Copy
 * Copy methods for images of various types. In addition to routines for
 * copying pixels of identical layout from one image to another, there are
 * copy routines for select channels as well as packed-planar conversions:
 * - Select channel to multi-channel copy. E.g. given a three-channel source
 *      and destination image one may copy the second channel of the source
 *      to the third channel of the destination.
 * - Single channel to multi-channel copy. E.g. given a single-channel source
 *      and a four-channel destination, one may copy the contents of the single-
 *      channel source to the second channel of the destination.
 * - Select channel to single-channel copy. E.g. given a three-channel source
 *      and a single-channel destination one may copy the third channel of the 
 *      source to the destination.
 * - Multi-channel to planar copy. These copy operations split a multi-channel
 *      image into a set of single-channel images.
 * - Planar image to multi-channel copy. These copy routines combine separate
 *      color-planes (single channel images) into a single multi-channel image.
 * 
 * </pre>
 */
///@{
/**
 * <pre>
 * 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit image copy, ignoring alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image copy, not affecting Alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image copy, not affecting Alpha channel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit complex image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116sc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16sc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16sc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit complex image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132sc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32sc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32sc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32sc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit floating point image copy, not affecting Alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Two-channel 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point complex image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit floating-point complex image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32fc_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32fc_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 8-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 8-bit unsigned image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_8u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit unsigned image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit unsigned image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16u_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 16-bit signed image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_16s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit signed image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit signed image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32s_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C1MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation three channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C3MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit float image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_C4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * \ref masked_operation four channel 32-bit float image copy, ignoring alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1AC4MRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jint nMaskStep)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiCopy_32f_AC4MR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_AC4MR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, nMaskStep=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, nMaskStep);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_AC4MR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, nMaskStep_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 8-bit unsigned image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 8-bit unsigned image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit signed image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit signed image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit unsigned image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 16-bit unsigned image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit signed image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit signed image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit float image copy for three-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Select-channel 32-bit float image copy for four-channel images.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref select_destination_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4CRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4CR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4CR(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4CR(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel to single-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel to single-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 8-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 16-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 16-bit unsigned image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 32-bit signed image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to three-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Single-channel to four-channel 32-bit float image copy.
 * @param pSrc \ref select_source_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C1C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C1C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C1C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit float packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C3P3R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* aDst_native[3];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C3P3R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit float packed to planar image copy.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param aDst Planar \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1C4P4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray aDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (aDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDst' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_C4P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_C4P4R(pSrc=%p, nSrcStep=%d, aDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, aDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* aDst_native[4];
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, aDst, aDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_C4P4R(pSrc_native, nSrcStep_native, aDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // aDst is an array of native pointers
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 8-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 8-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_18u_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_8u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_8u_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp8u* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_8u_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16u* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit unsigned planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116u_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16u_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16u_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16u* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16u_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 16-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16s* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 16-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_116s_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_16s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_16s_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp16s* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_16s_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32s* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit signed planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132s_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32s_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32s_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32s* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32s_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three-channel 32-bit float planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_P3C3R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32f* aSrc_native[3];
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_P3C3R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four-channel 32-bit float planar to packed image copy.
 * @param aSrc Planar \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopy_132f_1P4C4RNative(JNIEnv *env, jclass cls, jobjectArray aSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (aSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrc' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCopy_32f_P4C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopy_32f_P4C4R(aSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        aSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    const Npp32f* aSrc_native[4];
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, aSrc, aSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopy_32f_P4C4R(aSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    // aSrc is an array of native pointers
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Bit-Depth Conversion
 * Convert bit-depth up and down.
 *
 * The integer conversion methods do not involve any scaling. Conversions that reduce bit-depth saturate
 * values exceeding the reduced range to the range's maximum/minimum value.
 * When converting from floating-point values to integer values, a rounding mode can be specified. After rounding
 * to integer values the values get saturated to the destination data type's range.
 * </pre>
 */
///@{
/**
 * <pre>
 * 8-bit unsigned to 16-bit unsigned conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned to 8-bit unsigned conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned to 16-bit unsigned  conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned to 8-bit unsigned conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned to 16-bit unsigned conversion, not affecting Alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned to 8-bit unsigned conversion, not affecting Alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned to 16-bit signed conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed to 8-bit unsigned conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned to 16-bit signed conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed to 8-bit unsignedconversion, not affecting Alpha.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned to 16-bit signed conversion, not affecting Alpha.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u16s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed to 8-bit unsigned conversion, not affecting Alpha.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s8u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit singedto 32-bit floating point conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point to 16-bit conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned to 32-bit floating point conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned to 32-bit floating point conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point to 16-bit unsigned conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f16u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point to 8-bit unsigned conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eRoundMode Flag specifying how fractional float values are rounded to integer values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_132f8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint eRoundMode)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_32f8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eRoundMode is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_32f8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eRoundMode=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, eRoundMode);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode eRoundMode_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eRoundMode_native = (NppRoundMode)eRoundMode;

    // Native function call
    NppStatus jniResult_native = nppiConvert_32f8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, eRoundMode_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eRoundMode is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned to 32-bit signed conversion.
 * For detailed documentation see nppiConverte_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116u32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16u32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16u32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16u32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit to 32-bit  conversion.
 * For detailed documentation see nppiConvert_8u16u_C1R().
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiConvert_116s32s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiConvert_16s32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiConvert_16s32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiConvert_16s32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Copy Const Border
 * Methods for copying images and padding borders with a constant, user-specifiable color.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned image copy width constant border color.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region of pixels.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size (width, height) of the destination region, i.e. the region that gets filled with
 *      data from the source image (inner part) and constant border color (outer part).
 * @param nTopBorderHeight Height (in pixels) of the top border. The height of the border at the bottom of the
 *      destination ROI is implicitly defined by the size of the source ROI: 
 *      nBottomBorderHeight = oDstSizeROI.height - nTopBorderHeight - oSrcSizeROI.height.
 * @param nLeftBorderWidth Width (in pixels) of the left border. The width of the border at the right side of the
 *      destination ROI is implicitly defined by the size of the source ROI:
 *      nRightBorderWidth = oDstSizeROI.width - nLeftBorderWidth - oSrcSizeROI.width.
 * @param nValue The pixel value to be set for border pixels.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyte nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%c)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp8u)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4channel 8-bit unsigned image copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGBA values of the border pixels to be set.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyteArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_C4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u aValue_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_C4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param aValue Vector of the RGB values of the border pixels. Because this method does not
 *      affect the destination image's alpha channel, only three components of the border color
 *      are needed.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jbyteArray aValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (aValue == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aValue' is null for nppiCopyConstBorder_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_8u_AC4R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, aValue=%p)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, aValue);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp8u aValue_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    if (!initNative(env, aValue, aValue_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_8u_AC4R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, aValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    if (!releaseNative(env, aValue_native, aValue, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit image copy with constant border color.
 * See nppiCopyConstBorder_8u_C1R() for detailed documentation.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSizeROI Size of the source region-of-interest.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstSizeROI Size of the destination region-of-interest.
 * @param nTopBorderHeight Height of top border.
 * @param nLeftBorderWidth Width of left border.
 * @param nValue Border luminance value.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCopyConstBorder_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSrcSizeROI, jobject pDst, jint nDstStep, jobject oDstSizeROI, jint nTopBorderHeight, jint nLeftBorderWidth, jint nValue)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSizeROI' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstSizeROI' is null for nppiCopyConstBorder_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCopyConstBorder_32s_C1R(pSrc=%p, nSrcStep=%d, oSrcSizeROI=%p, pDst=%p, nDstStep=%d, oDstSizeROI=%p, nTopBorderHeight=%d, nLeftBorderWidth=%d, nValue=%d)\n",
        pSrc, nSrcStep, oSrcSizeROI, pDst, nDstStep, oDstSizeROI, nTopBorderHeight, nLeftBorderWidth, nValue);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSrcSizeROI_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oDstSizeROI_native;
    int nTopBorderHeight_native = 0;
    int nLeftBorderWidth_native = 0;
    Npp32s nValue_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcSizeROI, oSrcSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstSizeROI, oDstSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nTopBorderHeight_native = (int)nTopBorderHeight;
    nLeftBorderWidth_native = (int)nLeftBorderWidth;
    nValue_native = (Npp32s)nValue;

    // Native function call
    NppStatus jniResult_native = nppiCopyConstBorder_32s_C1R(pSrc_native, nSrcStep_native, oSrcSizeROI_native, pDst_native, nDstStep_native, oDstSizeROI_native, nTopBorderHeight_native, nLeftBorderWidth_native, nValue_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSrcSizeROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstSizeROI is a read-only structure
    // nTopBorderHeight is primitive
    // nLeftBorderWidth is primitive
    // nValue is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Image Transpose
 * Methods for transposing images of various types. Like matrix transpose, image transpose is a mirror along the image's
 * diagonal (upper-left to lower-right corner).
 * </pre>
 */
///@{
/**
 * <pre>
 * 8-bit image transpose.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst Pointer to the destination ROI.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 *
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiTranspose_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiTranspose_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiTranspose_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiTranspose_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Image Color Channel Swap
 * Methods for exchanging the color channels of an image. The methods support arbitrary permutations of the original
 * channels, including replication.
 * </pre>
 */
///@{
/**
 * <pre>
 * 4 channel 8-bit unsigned swap channels, in-place.
 *
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aDstOrder Integer array describing how channel values are permutated. The n-th entry
 *      of the array contains the number of the channel that is stored in the n-th channel of
 *      the output image. E.g. Given an RGBA image, aDstOrder = [3,2,1,0] converts this to ABGR
 *      channel order.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSwapChannels_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jintArray aDstOrder)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aDstOrder == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aDstOrder' is null for nppiSwapChannels_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSwapChannels_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, aDstOrder=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI, aDstOrder);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int aDstOrder_native[4];

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aDstOrder, aDstOrder_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSwapChannels_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, aDstOrder_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aDstOrder_native, aDstOrder, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
///@} image_data_exchange_and_initialization
/** @defgroup image_arithmetic Arithmetic and Logical Operations
 */
///@{
/** @name AddC
 * Adds a constant value to each pixel of an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel..
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel..
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel..
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_8u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_8u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16s_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16s_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_16sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_16sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image add constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image add constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image add constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32f_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32f_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image add constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image add constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddC_32fc_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddC_32fc_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AddC
///@}
/** @name MulC
 * Multiplies each pixel of an image by a constant value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_8u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_8u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16s_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16s_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_16sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_16sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiply by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiply by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image multiply by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32f_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32f_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image multiply by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image multiply by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulC_32fc_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulC_32fc_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of MulC
///@}
/** @name MulCScale
 * Multiplies each pixel of an image by a constant value then scales the result by the maximum value for the data bit width.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image multiply by constant and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiply by constant, scale and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiply by constant and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiply by constant and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiply by constant and scale by max bit width value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiply by constant and scale by max bit width value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulCScale_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulCScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulCScale_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulCScale_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of MulCScale
///@}
/** @name SubC
 * Subtracts a constant value from each pixel of an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_8u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_8u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16s_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16s_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_16sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_16sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtract constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtract constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image subtract constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32f_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32f_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image subtract constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image subtract constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSubC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSubC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSubC_32fc_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSubC_32fc_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of SubC
///@}
/** @name DivC
 * Divides each pixel of an image by a constant value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C1IRSfs(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel 8-bit unsigned char in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_8u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_8u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16u_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16u_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp16s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16s_C4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16s_C4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc nConstant_native;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc nConstant_native;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_16sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pConstants_native = NULL;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_16sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C1IRSfs(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32s_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32s_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc nConstant_native;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C1IRSfs(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc nConstant_native;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C1IRSfs(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_C3IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_C3IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha image divided by constant, scale, then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed complex integer (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image divided by constant, scale, then clamp to saturated value.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32sc_AC4IRSfs(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pConstants_native = NULL;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32sc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32sc_AC4IRSfs(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%f, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nConstant_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32f)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image divided by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C1IRNative(JNIEnv *env, jclass cls, jfloat nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C1IR(nConstant=%f, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f nConstant_native;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32f)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pConstants_native = NULL;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32f_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pConstants_native = NULL;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32f*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32f_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc nConstant_native;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (nConstant == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nConstant' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C1IR(nConstant=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc nConstant_native;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    if (!initNative(env, nConstant, nConstant_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releaseNative(env, nConstant_native, nConstant)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel with unmodified alpha in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel image divided by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit complex floating point (32-bit floating point real, 32-bit floating point imaginary) channel in place image divided by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDivC_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDivC_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDivC_32fc_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pConstants_native = NULL;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32fc*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDivC_32fc_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of DivC
///@}
/** @name AbsDiffC
 * Determines absolute difference between each pixel of an image and a constant value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image absolute difference with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jbyte nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_8u_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%c)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp8u)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_8u_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image absolute difference with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_16u_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%d)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32u nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp32u)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_16u_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image absolute difference with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiffC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nConstant)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiffC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nConstant is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiffC_32f_C1R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nConstant=%f)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI, nConstant);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nConstant_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nConstant_native = (Npp32f)nConstant;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiffC_32f_C1R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native, nConstant_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nConstant is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AbsDiffC
///@}
/** @name Add Image
 * Pixel by pixel addition of two images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image add.
 * Add the pixel values of corresponding pixels in the ROI and write them to the output image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image addition.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image addition.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAdd_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAdd_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAdd_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAdd_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Add Image
///@}
/** @name Add Square Image
 * Pixel by pixel addition of squared pixels from source image to floating point pixel values of destination image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_8u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_8u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_8u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_8u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_16u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_16u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_16u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_16u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared then added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared then added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddSquare_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddSquare_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddSquare_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddSquare_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Add Square Image
///@}
/** @name Add Product Image
 * Pixel by pixel addition of product of pixels from two source images to floating point pixel values of destination image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_8u32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_8u32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_8u32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_8u32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_16u32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_16u32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_16u32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_16u32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image product added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_32f_C1IMR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_32f_C1IMR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image product added to in place floating point destination image.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddProduct_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddProduct_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddProduct_32f_C1IR(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAddProduct_32f_C1IR(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Add Product Image
///@}
/** @name Add Weighted Image
 * Pixel by pixel addition of alpha weighted pixel values from a source image to floating point pixel values of destination image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_18u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_8u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_8u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_8u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_18u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_8u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_8u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_8u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_116u32f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_16u32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_16u32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_16u32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_116u32f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_16u32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_16u32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_16u32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel alpha weighted image added to in place floating point destination image using filter mask (updates destination when mask is non-zero).
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pMask \ref mask_image_pointer.
 * @param nMaskStep \ref mask_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_132f_1C1IMRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pMask, jint nMaskStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_32f_C1IMR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_32f_C1IMR(pSrc=%p, nSrcStep=%d, pMask=%p, nMaskStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pMask, nMaskStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pMask_native = NULL;
    int nMaskStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    nMaskStep_native = (int)nMaskStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_32f_C1IMR(pSrc_native, nSrcStep_native, pMask_native, nMaskStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel alpha weighted image added to in place floating point destination image.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nAlpha Alpha weight to be applied to source image pixels (0.0F to 1.0F)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAddWeighted_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jfloat nAlpha)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAddWeighted_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAlpha is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAddWeighted_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nAlpha=%f)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nAlpha);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nAlpha_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAlpha_native = (Npp32f)nAlpha;

    // Native function call
    NppStatus jniResult_native = nppiAddWeighted_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nAlpha_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nAlpha is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Add Weighted Image
///@}
/** @name Mul Image
 * Pixel by pixel multiply of two images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead.
 * 1 channel 32-bit image multiplication.
 * Multiply corresponding pixels in ROI. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiMul_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image multiplication.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image multiplication.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMul_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMul_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMul_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMul_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Mul Image
///@}
/** @name MulScale Image
 * Pixel by pixel multiplies each pixel of two images then scales the result by the maximum value for the data bit width.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image multiplication then scale by maximum value for pixel bit width.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMulScale_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMulScale_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMulScale_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMulScale_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of MulScale Image
///@}
/** @name Sub Image
 * Pixel by pixel subtraction of two images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image subtraction.
 * Subtract pSrc1's pixels from corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtraction, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSub_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image subtraction.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image subtraction.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSub_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSub_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSub_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSub_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Sub Image
///@}
/** @name Div Image
 * Pixel by pixel division of two images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short complex number (16-bit real, 16-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_116sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_16sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_16sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_16sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Note: This function is to be deprecated in future NPP releases, use the function above with a scale factor of 0 instead. 
 * 32-bit image division.
 * Divide pixels in pSrc2 by pSrc1's pixels. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32sc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32sc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32sc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32sc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32sc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132sc_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32sc_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32sc_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp32sc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32sc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32sc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32sc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32sc_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel with unmodified alpha in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32f_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32f_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel with unmodified alpha in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel image division.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32fc* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32fc* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32fc*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32fc*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32fc*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point complex number (32-bit real, 32-bit imaginary) channel in place image division.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_132fc_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_32fc_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_32fc_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32fc* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32fc*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDiv_32fc_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Div Image
///@}
/** @name Div_Round Image
 * Pixel by pixel division of two images using result rounding modes.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_8u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_8u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16u_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16u_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C1RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C1RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C1IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C1IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C3RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C3RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C3IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C3IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_AC4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_AC4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_AC4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_AC4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C4RSfs(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C4RSfs(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image division, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rndMode Result Rounding mode to be used (NPP_RND_ZERO, NPP_RND_NEAR, or NP_RND_FINANCIAL)
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDiv_1Round_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint rndMode, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDiv_Round_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // rndMode is primitive
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDiv_Round_16s_C4IRSfs(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, rndMode=%d, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI, rndMode, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppRoundMode rndMode_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    rndMode_native = (NppRoundMode)rndMode;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiDiv_Round_16s_C4IRSfs(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native, rndMode_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // rndMode is primitive
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Div_Round Image
///@}
/** @name Abs Image
 * Absolute value of each pixel value in an image.
 */
///@{
/**
 * <pre>
 * One 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image absolute value with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image absolute value with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_16s_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_16s_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image absolute value with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image absolute value with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image absolute value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image absolute value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbs_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAbs_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbs_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbs_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbs_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Abs Image
///@}
/** @name AbsDiff Image
 * Pixel by pixel absolute difference between two images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel absolute difference of image1 minus image2.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAbsDiff_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAbsDiff_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAbsDiff_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAbsDiff_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AbsDiff Image
///@}
/** @name Sqr Image
 * Square each pixel in an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_18u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_8u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_8u_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_8u_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116u_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16u_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16u_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16u_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image squared with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image squared, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_116s_1C4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_16s_C4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_16s_C4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqr_16s_C4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image squared.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image squared.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image squared with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image squared with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image squared.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image squared.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqr_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqr_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqr_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqr_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqr_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Sqr Image
///@}
/** @name Sqrt Image
 * Pixel by pixel square root of each pixel in an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_18u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_8u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_8u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_8u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116u_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16u_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16u_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16u_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image square root, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1AC4RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_AC4RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_AC4RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_AC4RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image square root with unmodified alpha, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_116s_1AC4IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_16s_AC4IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_16s_AC4IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_16s_AC4IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image square root with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image square root with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image square root.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel in place image square root.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrt_132f_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiSqrt_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSqrt_32f_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrt_32f_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSqrt_32f_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Sqrt Image
///@}
/** @name Ln Image
 * Pixel by pixel natural logarithm of each pixel in an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image natural logarithm, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiLn_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image natural logarithm.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image natural logarithm.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image natural logarithm.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image natural logarithm.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLn_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLn_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLn_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLn_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLn_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Ln Image
///@}
/** @name Exp Image
 * Exponential value of each pixel in an image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_18u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_8u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_8u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_8u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116u_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16u_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16u_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16u_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C1RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C1RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C1RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C1RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C1IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C1IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C1IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C1IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C3RSfsNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C3RSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C3RSfs(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C3RSfs(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image exponential, scale by 2^(-nScaleFactor), then clamp to saturated value.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nScaleFactor \ref integer_result_scaling.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_116s_1C3IRSfsNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI, jint nScaleFactor)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_16s_C3IRSfs");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nScaleFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_16s_C3IRSfs(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p, nScaleFactor=%d)\n",
        pSrcDst, nSrcDstStep, oSizeROI, nScaleFactor);

    // Native variable declarations
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;
    int nScaleFactor_native = 0;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nScaleFactor_native = (int)nScaleFactor;

    // Native function call
    NppStatus jniResult_native = nppiExp_16s_C3IRSfs(pSrcDst_native, nSrcDstStep_native, oSizeROI_native, nScaleFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure
    // nScaleFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image exponential.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel in place image exponential.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel image exponential.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit floating point channel in place image exponential.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiExp_132f_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiExp_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiExp_32f_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiExp_32f_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32f* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32f*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiExp_32f_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Exp Image
///@}
/** @name AndC Image
 * Pixel by pixel logical and of an image with a constant.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical and with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAndC_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAndC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAndC_32s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAndC_32s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AndC Image
///@}
/** @name OrC Image
 * Pixel by pixel logical or of an image with a constant.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOrC_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOrC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOrC_32s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOrC_32s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of OrC Image
///@}
/** @name XorC Image
 * Pixel by pixel logical exclusive or of an image with a constant.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp8u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C1IR(nConstant=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp8u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp8u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp16u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp16u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp16u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32s)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32s)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXorC_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXorC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXorC_32s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32s*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXorC_32s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of XorC Image
///@}
/** @name RShiftC Image
 * Pixel by pixel right shift of an image by a constant value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image right shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit signed char channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel in place image right shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit signed char channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_18s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_8s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_8s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_8s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image right shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit signed short channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image right shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit signed short channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_116s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_16s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_16s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_16s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image right shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image right shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image right shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image right shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image right shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRShiftC_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRShiftC_32s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRShiftC_32s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of RShiftC Image
///@}
/** @name LShiftC Image
 * Pixel by pixel left shift of an image by a constant value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image left shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_8u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_8u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image left shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_16u_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_16u_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nConstant Constant.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nConstant, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nConstant is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nConstant=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nConstant, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nConstant_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nConstant_native = (Npp32u)nConstant;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C1R(pSrc1_native, nSrc1Step_native, nConstant_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nConstant is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image left shift by constant.
 * @param nConstant Constant.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C1IRNative(JNIEnv *env, jclass cls, jint nConstant, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nConstant is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C1IR(nConstant=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nConstant, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u nConstant_native;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nConstant_native = (Npp32u)nConstant;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C1IR(nConstant_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nConstant is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C3R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C3R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C3IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C3IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image left shift by constant with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_AC4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image left shift by constant with unmodified alpha.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_AC4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_AC4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image left shift by constant.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pConstants, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C4R(pSrc1=%p, nSrc1Step=%d, pConstants=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pConstants, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pConstants_native = NULL;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C4R(pSrc1_native, nSrc1Step_native, pConstants_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image left shift by constant.
 * @param pConstants pointer to a list of constant values, one per channel.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLShiftC_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pConstants, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pConstants == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pConstants' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLShiftC_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLShiftC_32s_C4IR(pConstants=%p, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pConstants, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32u* pConstants_native = NULL;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pConstants_pointerData = initPointerData(env, pConstants);
    if (pConstants_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pConstants_native = (Npp32u*)pConstants_pointerData->getPointer(env);
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLShiftC_32s_C4IR(pConstants_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pConstants_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of LShiftC Image
///@}
/** @name And Image
 * Pixel by pixel logical and of images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical and.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical and.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAnd_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAnd_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAnd_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAnd_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of And Image
///@}
/** @name Or Image
 * Pixel by pixel logical or of images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiOr_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiOr_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiOr_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiOr_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Or Image
///@}
/** @name Xor Image
 * Pixel by pixel logical exclusive or of images.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_8u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_8u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_116u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_16u_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_16u_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C1IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C1IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C3R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C3R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C3IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C3IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or with unmodified alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_AC4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_AC4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image logical exclusive or.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel in place image logical exclusive or.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiXor_132s_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiXor_32s_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiXor_32s_C4IR(pSrc=%p, nSrcStep=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp32s*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiXor_32s_C4IR(pSrc_native, nSrcStep_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Xor Image
///@}
/** @name Not Image
 * Pixel by pixel logical not of image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C1IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C1IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C1IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C3IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C3IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C3IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical not with unmodified alpha.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical not with unmodified alpha.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image logical not.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image logical not.
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNot_18u_1C4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiNot_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNot_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNot_8u_C4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiNot_8u_C4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of Not Image
///@}
/** @name AlphaCompC Image
 * Composite two images using constant alpha values.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image composition with alpha using constant source alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u nAlpha2_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_18s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pSrc2, jint nSrc2Step, jbyte nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_8s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_8s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pSrc2=%p, nSrc2Step=%d, nAlpha2=%c, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s nAlpha1_native;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8s nAlpha2_native;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp8s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_8s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition with alpha using constant source alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u nAlpha2_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pSrc2, jint nSrc2Step, jshort nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_16s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s nAlpha1_native;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s nAlpha2_native;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp16s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_16s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit unsigned integer channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nAlpha1, jobject pSrc2, jint nSrc2Step, jint nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u nAlpha1_native;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u nAlpha2_native;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32u)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32u)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jint nAlpha1, jobject pSrc2, jint nSrc2Step, jint nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32s_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pSrc2=%p, nSrc2Step=%d, nAlpha2=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s nAlpha1_native;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s nAlpha2_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32s)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32s)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32s_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image composition using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0.0 - 1.0).
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param nAlpha2 Image alpha opacity (0.0 - 1.0).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaCompC_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jfloat nAlpha1, jobject pSrc2, jint nSrc2Step, jfloat nAlpha2, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaCompC_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaCompC_32f_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%f, pSrc2=%p, nSrc2Step=%d, nAlpha2=%f, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, nAlpha1, pSrc2, nSrc2Step, nAlpha2, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f nAlpha1_native;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f nAlpha2_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp32f)nAlpha1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    nAlpha2_native = (Npp32f)nAlpha2;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaCompC_32f_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pSrc2_native, nSrc2Step_native, nAlpha2_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    // nAlpha2 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AlphaCompC Image
///@}
/** @name AlphaPremulC Image
 * Premultiplies pixels of an image using a constant alpha value.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C1IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C1IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C1IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C3IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C3IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C3IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1C4IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_C4IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_C4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication with alpha using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jbyte nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%c, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u nAlpha1_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication with alpha using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_18u_1AC4IRNative(JNIEnv *env, jclass cls, jbyte nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_8u_AC4IR(nAlpha1=%c, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u nAlpha1_native;
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp8u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_8u_AC4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C1R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C1R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C1IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C1IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C1IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C1IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C3R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C3R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Three 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C3IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C3IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C3IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C3IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image premultiplication using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1C4IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_C4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_C4IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_C4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image premultiplication with alpha using constant alpha.
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jshort nAlpha1, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_AC4R(pSrc1=%p, nSrc1Step=%d, nAlpha1=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, nAlpha1, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u nAlpha1_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_AC4R(pSrc1_native, nSrc1Step_native, nAlpha1_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    // nAlpha1 is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication with alpha using constant alpha.
 * @param nAlpha1 Image alpha opacity (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremulC_116u_1AC4IRNative(JNIEnv *env, jclass cls, jshort nAlpha1, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    // nAlpha1 is primitive
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremulC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremulC_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremulC_16u_AC4IR(nAlpha1=%d, pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        nAlpha1, pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u nAlpha1_native;
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    nAlpha1_native = (Npp16u)nAlpha1;
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremulC_16u_AC4IR(nAlpha1_native, pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    // nAlpha1 is primitive
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AlphaPremulC Image
///@}
/** @name AlphaComp Image
 * Composite two images using alpha opacity values contained in each image.
 */
///@{
/**
 * <pre>
 * One 8-bit unsigned char channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 8-bit signed char channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_18s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_8s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_8s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp8s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_8s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 16-bit signed short channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_116s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_16s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_16s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp16s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp16s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_16s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132u_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32u_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32u_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32u_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit unsigned integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132s_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32s_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32s_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32s_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit signed integer channel image composition using image alpha values (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32s_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32s* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32s* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32s*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32s*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32s_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * One 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132f_1AC1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32f_AC1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32f_AC1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32f_AC1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 32-bit floating point channel image composition using image alpha values (0.0 - 1.0).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eAlphaOp alpha-blending operation..
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaComp_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eAlphaOp)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaComp_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eAlphaOp is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaComp_32f_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eAlphaOp=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eAlphaOp);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiAlphaOp eAlphaOp_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eAlphaOp_native = (NppiAlphaOp)eAlphaOp;

    // Native function call
    NppStatus jniResult_native = nppiAlphaComp_32f_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eAlphaOp_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eAlphaOp is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AlphaComp Image
///@}
/** @name AlphaPremul Image
 * Premultiplies image pixels by image alpha opacity values.
 */
///@{
/**
 * <pre>
 * Four 8-bit unsigned char channel image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_8u_AC4R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 8-bit unsigned char channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_18u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremul_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_8u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_8u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp8u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_8u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_16u_AC4R(pSrc1=%p, nSrc1Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc1, nSrc1Step, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp16u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_16u_AC4R(pSrc1_native, nSrc1Step_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Four 16-bit unsigned short channel in place image premultiplication with pixel alpha (0 - max channel pixel value).
 * @param pSrcDst \ref in_place_image_pointer.
 * @param nSrcDstStep \ref in_place_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiAlphaPremul_116u_1AC4IRNative(JNIEnv *env, jclass cls, jobject pSrcDst, jint nSrcDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrcDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrcDst' is null for nppiAlphaPremul_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiAlphaPremul_16u_AC4IR");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiAlphaPremul_16u_AC4IR(pSrcDst=%p, nSrcDstStep=%d, oSizeROI=%p)\n",
        pSrcDst, nSrcDstStep, oSizeROI);

    // Native variable declarations
    Npp16u* pSrcDst_native = NULL;
    int nSrcDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrcDst_pointerData = initPointerData(env, pSrcDst);
    if (pSrcDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrcDst_native = (Npp16u*)pSrcDst_pointerData->getPointer(env);
    nSrcDstStep_native = (int)nSrcDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiAlphaPremul_16u_AC4IR(pSrcDst_native, nSrcDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrcDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

// end of AlphaPremul Image
///@}
// end of Image Arithmetic
///@}
/**
 * <pre>
 * @defgroup image_threshold_and_compare_operations Threshold and Compare Operations
 *
 * Methods for pixel-wise threshold and compare operations.
 * </pre>
 */
///@{
/**
 * <pre>
 * @name Threshold
 *
 * Threshold pixels.
 * </pre>
 */
///@{
/**
 * <pre>
 * 32-bit floating point threshold.
 * If for a comparison operations OP the predicate (sourcePixel OP nThreshold) is true, the pixel is set
 * to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param nThreshold The threshold value.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jfloat nThreshold, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, nThreshold=%f, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, nThreshold, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f nThreshold_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nThreshold_native = (Npp32f)nThreshold;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, nThreshold_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // nThreshold is primitive
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image threshold, not affecting Alpha.
 * If for a comparison operations OP the predicate (sourcePixel.channel OP nThreshold) is true, the channel
 * value is set to nThreshold, otherwise it is set to sourcePixel.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aThresholds The threshold values, one per color channel.
 * @param eComparisonOperation The type of comparison operation to be used. The only valid
 *      values are: NPP_CMP_LESS and NPP_CMP_GREATER.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, or NPP_NOT_SUPPORTED_MODE_ERROR if an invalid
 * comparison operation type is specified.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiThreshold_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jbyteArray aThresholds, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aThresholds == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aThresholds' is null for nppiThreshold_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiThreshold_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, aThresholds=%p, eComparisonOperation=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, aThresholds, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u aThresholds_native[3];
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aThresholds, aThresholds_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiThreshold_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, aThresholds_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aThresholds_native, aThresholds, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Image Compare Methods
 * Compare the pixels of two images and create a binary result image. In case of multi-channel
 * image types, the condition must be fulfilled for all channels, otherwise the comparison
 * is considered false.
 * The "binary" result image is of type 8u_C1. False is represented by 0, true by NPP_MAX_8U.
 * </pre>
 */
///@{
/**
 * <pre>
 * 4 channel 8-bit unsigned image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_C4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_C4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image compare, not affecting Alpha.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_8u_AC4R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_8u_AC4R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point image compare.
 * Compare pSrc1's pixels with corresponding pixels in pSrc2. 
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrc1Step \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrc2Step \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param eComparisonOperation Specifies the comparison operation to be used in the pixel comparison.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiCompare_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrc1Step, jobject pSrc2, jint nSrc2Step, jobject pDst, jint nDstStep, jobject oSizeROI, jint eComparisonOperation)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc1Step is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrc2Step is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiCompare_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eComparisonOperation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiCompare_32f_C1R(pSrc1=%p, nSrc1Step=%d, pSrc2=%p, nSrc2Step=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, eComparisonOperation=%d)\n",
        pSrc1, nSrc1Step, pSrc2, nSrc2Step, pDst, nDstStep, oSizeROI, eComparisonOperation);

    // Native variable declarations
    Npp32f* pSrc1_native = NULL;
    int nSrc1Step_native = 0;
    Npp32f* pSrc2_native = NULL;
    int nSrc2Step_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppCmpOp eComparisonOperation_native;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp32f*)pSrc1_pointerData->getPointer(env);
    nSrc1Step_native = (int)nSrc1Step;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp32f*)pSrc2_pointerData->getPointer(env);
    nSrc2Step_native = (int)nSrc2Step;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eComparisonOperation_native = (NppCmpOp)eComparisonOperation;

    // Native function call
    NppStatus jniResult_native = nppiCompare_32f_C1R(pSrc1_native, nSrc1Step_native, pSrc2_native, nSrc2Step_native, pDst_native, nDstStep_native, oSizeROI_native, eComparisonOperation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc1Step is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrc2Step is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // eComparisonOperation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
///@} image_threshold_and_compare_operations
/**
 * <pre>
 * @defgroup image_statistics_functions Statistics Functions
 *
 * Routines computing statistical image information.
 *
 *
 * </pre>
 */
///@{
/** @name Mean_StdDev
 *  Computes the mean and standard deviation of image pixel values
 */
///@{
/**
 * <pre>
 * Device scratch buffer size (in bytes) for mean and standard deviation of image.
 * This primitive provides the correct buffer size for nppiMean_StdDev_8u_C1R.
 * @param oSizeROI \ref roi_specification.
 * @param hpBufferSize Required buffer size. Important: hpBufferSize is a 
 *        <em>host pointer.</em> \ref general_scratch_buffer.
 * @return
 *  NPP_SUCCESS
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMeanStdDev8uC1RGetBufferHostSizeNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMeanStdDev8uC1RGetBufferHostSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMeanStdDev8uC1RGetBufferHostSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMeanStdDev8uC1RGetBufferHostSize(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMeanStdDev8uC1RGetBufferHostSize(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned mean standard deviation.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 * @param pMean Contains computed mean.
 * @param pStdDev Contains computed standard deviation.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMean_1StdDev_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray pMean, jdoubleArray pStdDev)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMean == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMean' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pStdDev == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pStdDev' is null for nppiMean_StdDev_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMean_StdDev_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pMean=%p, pStdDev=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pMean, pStdDev);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f* pMean_native = NULL;
    Npp64f* pStdDev_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    PointerData *pMean_pointerData = initPointerData(env, pMean);
    if (pMean_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMean_native = (Npp64f*)pMean_pointerData->getPointer(env);
    PointerData *pStdDev_pointerData = initPointerData(env, pStdDev);
    if (pStdDev_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pStdDev_native = (Npp64f*)pStdDev_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMean_StdDev_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, pMean_native, pStdDev_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMean_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pStdDev_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name NormDiff
 *  Norm of pixel differences between two images.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned L1 norm of pixel differences.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrcStep1 \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrcStep2 \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pRetVal Contains computed L1-norm of differences. This is a host pointer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L1_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrcStep1, jobject pSrc2, jint nSrcStep2, jobject oSizeROI, jdoubleArray pRetVal)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep2 is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRetVal == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRetVal' is null for nppiNormDiff_L1_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L1_8u_C1R(pSrc1=%p, nSrcStep1=%d, pSrc2=%p, nSrcStep2=%d, oSizeROI=%p, pRetVal=%p)\n",
        pSrc1, nSrcStep1, pSrc2, nSrcStep2, oSizeROI, pRetVal);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrcStep1_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrcStep2_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pRetVal_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrcStep1_native = (int)nSrcStep1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrcStep2_native = (int)nSrcStep2;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pRetVal_pointerData = initPointerData(env, pRetVal);
    if (pRetVal_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRetVal_native = (Npp64f*)pRetVal_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L1_8u_C1R(pSrc1_native, nSrcStep1_native, pSrc2_native, nSrcStep2_native, oSizeROI_native, pRetVal_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep2 is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pRetVal_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned L2 norm of pixel differences.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrcStep1 \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrcStep2 \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pRetVal Contains computed L1-norm of differences. This is a host pointer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1L2_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrcStep1, jobject pSrc2, jint nSrcStep2, jobject oSizeROI, jdoubleArray pRetVal)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep2 is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRetVal == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRetVal' is null for nppiNormDiff_L2_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_L2_8u_C1R(pSrc1=%p, nSrcStep1=%d, pSrc2=%p, nSrcStep2=%d, oSizeROI=%p, pRetVal=%p)\n",
        pSrc1, nSrcStep1, pSrc2, nSrcStep2, oSizeROI, pRetVal);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrcStep1_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrcStep2_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pRetVal_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrcStep1_native = (int)nSrcStep1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrcStep2_native = (int)nSrcStep2;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pRetVal_pointerData = initPointerData(env, pRetVal);
    if (pRetVal_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRetVal_native = (Npp64f*)pRetVal_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_L2_8u_C1R(pSrc1_native, nSrcStep1_native, pSrc2_native, nSrcStep2_native, oSizeROI_native, pRetVal_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep2 is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pRetVal_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned Infinity Norm of pixel differences.
 *
 * @param pSrc1 \ref source_image_pointer.
 * @param nSrcStep1 \ref source_image_line_step.
 * @param pSrc2 \ref source_image_pointer.
 * @param nSrcStep2 \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param *pRetVal Contains computed L1-norm of differences. This is a host pointer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiNormDiff_1Inf_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc1, jint nSrcStep1, jobject pSrc2, jint nSrcStep2, jobject oSizeROI, jdoubleArray pRetVal)
{
    // Null-checks for non-primitive arguments
    if (pSrc1 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc1' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep1 is primitive
    if (pSrc2 == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc2' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep2 is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRetVal == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRetVal' is null for nppiNormDiff_Inf_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiNormDiff_Inf_8u_C1R(pSrc1=%p, nSrcStep1=%d, pSrc2=%p, nSrcStep2=%d, oSizeROI=%p, pRetVal=%p)\n",
        pSrc1, nSrcStep1, pSrc2, nSrcStep2, oSizeROI, pRetVal);

    // Native variable declarations
    Npp8u* pSrc1_native = NULL;
    int nSrcStep1_native = 0;
    Npp8u* pSrc2_native = NULL;
    int nSrcStep2_native = 0;
    NppiSize oSizeROI_native;
    Npp64f* pRetVal_native = NULL;

    // Obtain native variable values
    PointerData *pSrc1_pointerData = initPointerData(env, pSrc1);
    if (pSrc1_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc1_native = (Npp8u*)pSrc1_pointerData->getPointer(env);
    nSrcStep1_native = (int)nSrcStep1;
    PointerData *pSrc2_pointerData = initPointerData(env, pSrc2);
    if (pSrc2_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc2_native = (Npp8u*)pSrc2_pointerData->getPointer(env);
    nSrcStep2_native = (int)nSrcStep2;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pRetVal_pointerData = initPointerData(env, pRetVal);
    if (pRetVal_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRetVal_native = (Npp64f*)pRetVal_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiNormDiff_Inf_8u_C1R(pSrc1_native, nSrcStep1_native, pSrc2_native, nSrcStep2_native, oSizeROI_native, pRetVal_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc1_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep1 is primitive
    if (!releasePointerData(env, pSrc2_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep2 is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pRetVal_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * SqrIntegral Transforms an image to integral and integral of pixel squares
 * representation. This function assumes that the integral and integral of squares 
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param oSrcROI \ref roi_specification.
 * @param val The value to add to pDst image pixels
 * @param valSqr The value to add to pSqr image pixels
 * @param integralImageNewHeight Extended height of output surfaces (needed by
 *        transpose in primitive)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSqrIntegral_18u32s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pSqr, jint nSqrStep, jobject oSrcROI, jint val, jfloat valSqr, jint integralImageNewHeight)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSqrIntegral_8u32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSqrIntegral_8u32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiSqrIntegral_8u32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiSqrIntegral_8u32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // val is primitive
    // valSqr is primitive
    // integralImageNewHeight is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSqrIntegral_8u32s32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pSqr=%p, nSqrStep=%d, oSrcROI=%p, val=%d, valSqr=%f, integralImageNewHeight=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, pSqr, nSqrStep, oSrcROI, val, valSqr, integralImageNewHeight);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp32f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    NppiSize oSrcROI_native;
    Npp32s val_native;
    Npp32f valSqr_native;
    Npp32s integralImageNewHeight_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp32f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    val_native = (Npp32s)val;
    valSqr_native = (Npp32f)valSqr;
    integralImageNewHeight_native = (Npp32s)integralImageNewHeight;

    // Native function call
    NppStatus jniResult_native = nppiSqrIntegral_8u32s32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pSqr_native, nSqrStep_native, oSrcROI_native, val_native, valSqr_native, integralImageNewHeight_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    // oSrcROI is a read-only structure
    // val is primitive
    // valSqr is primitive
    // integralImageNewHeight is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * RectStdDev Computes the standard deviation of integral images
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pSqr \ref destination_image_pointer.
 * @param nSqrStep \ref destination_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param rect rectangular window
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRectStdDev_132s32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jdoubleArray pSqr, jint nSqrStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject rect)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pSqr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSqr' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSqrStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (rect == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'rect' is null for nppiRectStdDev_32s32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRectStdDev_32s32f_C1R(pSrc=%p, nSrcStep=%d, pSqr=%p, nSqrStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, rect=%p)\n",
        pSrc, nSrcStep, pSqr, nSqrStep, pDst, nDstStep, oSizeROI, rect);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp64f* pSqr_native = NULL;
    int nSqrStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    NppiRect rect_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pSqr_pointerData = initPointerData(env, pSqr);
    if (pSqr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSqr_native = (Npp64f*)pSqr_pointerData->getPointer(env);
    nSqrStep_native = (int)nSqrStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, rect, rect_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRectStdDev_32s32f_C1R(pSrc_native, nSrcStep_native, pSqr_native, nSqrStep_native, pDst_native, nDstStep_native, oSizeROI_native, rect_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pSqr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSqrStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // rect is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/** @name Sum
 *  Sum of 8 bit images.
 */
///@{
/**
 * <pre>
 * Scratch-buffer size for nppiSum_8u_C1R.
 * 
 * @param oSizeROI ROI size.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiReductionGetBufferHostSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiReductionGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiReductionGetBufferHostSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiReductionGetBufferHostSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiReductionGetBufferHostSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiSum_8u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiReductionGetBufferHostSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiReductionGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiReductionGetBufferHostSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiReductionGetBufferHostSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiReductionGetBufferHostSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image sum with 64-bit long long result.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 * @param *pSum Contains computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u64s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jlongArray pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_8u64s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u64s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64s pSum_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    // pSum is set here

    // Native function call
    NppStatus jniResult_native = nppiSum_8u64s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, &pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, pSum, 0, (jlong)pSum_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image sum with 64-bit double precision result.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 * @param *pSum Contains computed sum.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray pSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSum' is null for nppiSum_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, pSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, pSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f pSum_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    // pSum is set here

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, &pSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, pSum, 0, (jdouble)pSum_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image sum with 64-bit long long result.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 * @param aSum Array contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u64s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jlongArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u64s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u64s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64s aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u64s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image sum with 64-bit double precision result.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pDeviceBuffer Pointer to the required device memory allocation, \ref general_scratch_buffer
 * @param aSum Array contains computed sum for each channel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSum_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pDeviceBuffer, jdoubleArray aSum)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSum == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSum' is null for nppiSum_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSum_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pDeviceBuffer=%p, aSum=%p)\n",
        pSrc, nSrcStep, oSizeROI, pDeviceBuffer, aSum);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pDeviceBuffer_native = NULL;
    Npp64f aSum_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);
    if (!initNative(env, aSum, aSum_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiSum_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pDeviceBuffer_native, aSum_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSum_native, aSum, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name MinMax
 *  Minimum and maximum of 8-bit images.
 */
///@{
/**
 * <pre>
 * Scratch-buffer size for nppiMinManx_8u_C1R.
 * 
 * @param oSizeROI ROI size.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferSize_8u_C1R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferSize_8u_C1R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned pixel minimum and maximum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMin Device-memory pointer receiving the minimum result.
 * @param pMax Device-memory pointer receiving the maximum result.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferSize_8u_C1R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pMin, jobject pMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMin' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMax' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pMin=%p, pMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pMin, pMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMin_native = NULL;
    Npp8u* pMax_native = NULL;
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMin_pointerData = initPointerData(env, pMin);
    if (pMin_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMin_native = (Npp8u*)pMin_pointerData->getPointer(env);
    PointerData *pMax_pointerData = initPointerData(env, pMax);
    if (pMax_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMax_native = (Npp8u*)pMax_pointerData->getPointer(env);
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pMin_native, pMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMin_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pMax_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiMinManx_8u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMaxGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMaxGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiMinMaxGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMaxGetBufferSize_8u_C4R(oSizeROI=%p, hpBufferSize=%p)\n",
        oSizeROI, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiMinMaxGetBufferSize_8u_C4R(oSizeROI_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned pixel minimum and maximum.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param aMin Device-pointer (array) receiving the minimum result.
 * @param aMax Device-pointer (array) receiving the maximum result.
 * @param pDeviceBuffer Buffer to a scratch memory. Use \ref nppiMinMaxGetBufferSize_8u_C4R to determine
 *          the minium number of bytes required.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *
 * \note Unlike nppiMinMax_8u_C1R, this primitive returns its results as device
 *       pointers.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMinMax_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jbyteArray aMin, jbyteArray aMax, jobject pDeviceBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMin == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMin' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aMax == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aMax' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceBuffer' is null for nppiMinMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMinMax_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, aMin=%p, aMax=%p, pDeviceBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, aMin, aMax, pDeviceBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u aMin_native[4];
    Npp8u aMax_native[4];
    Npp8u* pDeviceBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMin, aMin_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aMax, aMax_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDeviceBuffer_pointerData = initPointerData(env, pDeviceBuffer);
    if (pDeviceBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceBuffer_native = (Npp8u*)pDeviceBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiMinMax_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, aMin_native, aMax_native, pDeviceBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, aMin_native, aMin, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aMax_native, aMax, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Histogram
 */
///@{
/**
 * <pre>
 * Compute levels with even distribution.
 *
 * @param hpLevels A host pointer to array which receives the levels being
 *        computed. The array needs to be of size nLevels.
 * @param nLevels The number of levels being computed. nLevels must be at least
 *        2, otherwise an NPP_HISTO_NUMBER_OF_LEVELS_ERROR error is returned.
 * @param nLowerLevel Lower boundary value of the lowest level.
 * @param nUpperLevel Upper boundary value of the greatest level.
 * @return
 *  Error code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiEvenLevelsHost_132sNative(JNIEnv *env, jclass cls, jobject hpLevels, jint nLevels, jint nLowerLevel, jint nUpperLevel)
{
    // Null-checks for non-primitive arguments
    if (hpLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpLevels' is null for nppiEvenLevelsHost_32s");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiEvenLevelsHost_32s(hpLevels=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d)\n",
        hpLevels, nLevels, nLowerLevel, nUpperLevel);

    // Native variable declarations
    Npp32s* hpLevels_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;

    // Obtain native variable values
    PointerData *hpLevels_pointerData = initPointerData(env, hpLevels);
    if (hpLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpLevels_native = (Npp32s*)hpLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;

    // Native function call
    NppStatus jniResult_native = nppiEvenLevelsHost_32s(hpLevels_native, nLevels_native, nLowerLevel_native, nUpperLevel_native);

    // Write back native variable values
    if (!releasePointerData(env, hpLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_8u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_8u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_8u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_8u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_8u_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_8u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_8u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as the last channel) 8-bit unsigned histogram with evenly distributed bins.
 * Alpha channel is ignored during histogram computation.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_8u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16u_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as the last channel) 16-bit unsigned histogram with evenly distributed bins.
 * Alpha channel is ignored during histogram computation.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16s_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param nLevels Number of levels.
 * @param nLowerLevel Lower boundary of lowest level bin.
 * @param nUpperLevel Upper boundary of highest level bin.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16s_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jint nLevels, jint nLowerLevel, jint nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%d, nLowerLevel=%d, nUpperLevel=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    int nLevels_native = 0;
    Npp32s nLowerLevel_native;
    Npp32s nUpperLevel_native;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    nLowerLevel_native = (Npp32s)nLowerLevel;
    nUpperLevel_native = (Npp32s)nUpperLevel;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    // nLowerLevel is primitive
    // nUpperLevel is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16s_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed histogram with evenly distributed bins.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16s_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    int nLevels_native[4];
    Npp32s nLowerLevel_native[4];
    Npp32s nUpperLevel_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramEven_16s_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEvenGetBufferSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramEvenGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEvenGetBufferSize_16s_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramEvenGetBufferSize_16s_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as the last channel) 16-bit signed histogram with evenly distributed bins.
 * Alpha channel is ignored during histogram computation.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving computed histograms per color channel. 
 *      Array pointed by pHist[i] be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param nLowerLevel Array containing lower-level of lowest bin per color channel.
 * @param nUpperLevel Array containing upper-level of highest bin per color channel.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramEvenGetBufferSize_16s_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramEven_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jintArray nLevels, jintArray nLowerLevel, jintArray nUpperLevel, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLowerLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLowerLevel' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nUpperLevel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nUpperLevel' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramEven_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramEven_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, nLevels=%p, nLowerLevel=%p, nUpperLevel=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, nLevels, nLowerLevel, nUpperLevel, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    int nLevels_native[3];
    Npp32s nLowerLevel_native[3];
    Npp32s nUpperLevel_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLowerLevel, nLowerLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nUpperLevel, nUpperLevel_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramEven_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, nLevels_native, nLowerLevel_native, nUpperLevel_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nLowerLevel_native, nLowerLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nUpperLevel_native, nUpperLevel, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned histogram with bins determined by pLevels array.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_8u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned histogram with bins determined by pLevels.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_8u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_8u_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_8u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_8u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as a last channel) 8-bit unsigned histogram with bins determined by pLevels.
 * Alpha channel is ignored during the histograms computations.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_8u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_8u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_8u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned histogram with bins determined by pLevels array.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16u_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned histogram with bins determined by pLevels.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16u_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16u_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16u_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16u_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as a last channel) 16-bit unsigned histogram with bins determined by pLevels.
 * Alpha channel is ignored during the histograms computations.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16u_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16u_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16u_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit signed histogram with bins determined by pLevels array.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit signed histogram with bins determined by pLevels.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16s_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_16s_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_16s_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_16s_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as a last channel) 16-bit signed histogram with bins determined by pLevels.
 * Alpha channel is ignored during the histograms computations.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_16_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_116s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_16s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_16s_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_16s_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_C1R.
 * 
 * @param oSizeROI \ref roi_specification.
 * @param nLevels Number of levels in the histogram.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1C1RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jint nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_C1R(oSizeROI=%p, nLevels=%d, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native = 0;
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nLevels_native = (int)nLevels;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_C1R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    // nLevels is primitive
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float histogram with bins determined by pLevels array.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Pointer to array that receives the computed histogram. 
 *      The array must be of size nLevels-1. 
 * @param pLevels Pointer to array containing the level sizes of the bins.
        The array must be of size nLevels.
 * @param nLevels Number of levels in histogram.
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C1R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobject pHist, jobject pLevels, jint nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_C1R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%d, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native = NULL;
    Npp32f* pLevels_native = NULL;
    int nLevels_native = 0;
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pHist_pointerData = initPointerData(env, pHist);
    if (pHist_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pHist_native = (Npp32s*)pHist_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32f*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_C1R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pHist_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_C4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1C4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_C4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[4];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_C4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float histogram with bins determined by pLevels.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_C4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_C4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[4];
    const Npp32f* pLevels_native[4];
    int nLevels_native[4];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_C4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Scratch-buffer size for nppiHistogramRange_32f_AC4R.
 * 
 * @param oSizeROI ROI size.
 * @param nLevels Array containing number of levels per color channel.
 * @param hpBufferSize Host pointer where required buffer size is returned.
 * @return
 *  Error Code.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRangeGetBufferSize_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject oSizeROI, jintArray nLevels, jintArray hpBufferSize)
{
    // Null-checks for non-primitive arguments
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpBufferSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpBufferSize' is null for nppiHistogramRangeGetBufferSize_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRangeGetBufferSize_32f_AC4R(oSizeROI=%p, nLevels=%p, hpBufferSize=%p)\n",
        oSizeROI, nLevels, hpBufferSize);

    // Native variable declarations
    NppiSize oSizeROI_native;
    int nLevels_native[3];
    int hpBufferSize_native = 0;

    // Obtain native variable values
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // hpBufferSize is set here

    // Native function call
    NppStatus jniResult_native = nppiHistogramRangeGetBufferSize_32f_AC4R(oSizeROI_native, nLevels_native, &hpBufferSize_native);

    // Write back native variable values
    // oSizeROI is a read-only structure
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!set(env, hpBufferSize, 0, (jint)hpBufferSize_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel (alpha as a last channel) 32-bit float histogram with bins determined by pLevels.
 * Alpha channel is ignored during the histograms computations.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pHist Array of pointers which are receiving the computed histograms per color channel. 
 *      Array pointed by pHist[i] must be of size nLevels[i]-1.
 * @param nLevels Array containing number of levels per color channel. 
 * @param pLevels Array containing pointers to level-arrays per color channel.
        Array pointed by pLevel[i] must be of size nLevels[i].
 * @param pBuffer Pointer to appropriately sized (nppiHistogramRangeGetBufferSize_32f_AC4R) 
 *      scratch buffer.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiHistogramRange_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject oSizeROI, jobjectArray pHist, jobjectArray pLevels, jintArray nLevels, jobject pBuffer)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pHist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pHist' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBuffer == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBuffer' is null for nppiHistogramRange_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiHistogramRange_32f_AC4R(pSrc=%p, nSrcStep=%d, oSizeROI=%p, pHist=%p, pLevels=%p, nLevels=%p, pBuffer=%p)\n",
        pSrc, nSrcStep, oSizeROI, pHist, pLevels, nLevels, pBuffer);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pHist_native[3];
    const Npp32f* pLevels_native[3];
    int nLevels_native[3];
    Npp8u* pBuffer_native = NULL;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pHist, pHist_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBuffer_pointerData = initPointerData(env, pBuffer);
    if (pBuffer_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBuffer_native = (Npp8u*)pBuffer_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiHistogramRange_32f_AC4R(pSrc_native, nSrcStep_native, oSizeROI_native, pHist_native, pLevels_native, nLevels_native, pBuffer_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    // oSizeROI is a read-only structure
    // pHist is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBuffer_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} Histogram group
///@} image_statistics_functions
/**
 * <pre>
 * @defgroup image_filtering_functions Filtering Functions
 *
 * Linear and non-linear image filtering functions.
 *
 *
 * </pre>
 */
///@{
/** @name 1D Linear Filter
 *  1D mask Linear Convolution Filter, with rescaling, for 8 bit images.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned 1D (column) image convolution.
 * 
 * Apply convolution filter with user specified 1D column of weights.  
 * Result pixel is equal to the sum of the products between the kernel
 * coefficients (pKernel array) and corresponding neighboring column pixel
 * values in the source image defined by nKernelDim and nAnchorY, divided by
 * nDivisor. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference w.r.t the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned 1D (column) image convolution.
 * 
 * Apply convolution filter with user specified 1D column of weights.  
 * Result pixel is equal to the sum of the products between the kernel
 * coefficients (pKernel array) and corresponding neighboring column pixel
 * values in the source image defined by nKernelDim and nAnchorY, divided by
 * nDivisor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference w.r.t the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterColumn_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterColumn_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterColumn_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterColumn_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned 1D (row) image convolution.
 *
 * Apply general linear Row convolution filter, with rescaling, in a 1D mask
 * region around each source pixel for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the sum of the products between the kernel
 * coefficients (pKernel array) and corresponding neighboring row pixel values
 * in the source image defined by iKernelDim and iAnchorX, divided by iDivisor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.  
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference w.r.t the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned 1D (row) image convolution.
 *
 * Apply general linear Row convolution filter, with rescaling, in a 1D mask
 * region around each source pixel for 1-channel 8 bit/pixel images.  
 * Result pixel is equal to the sum of the products between the kernel
 * coefficients (pKernel array) and corresponding neighboring row pixel values
 * in the source image defined by iKernelDim and iAnchorX, divided by iDivisor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *                Coefficients are expected to be stored in reverse order.  
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference w.r.t the
 *                 source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *                 operation should be divided.  If equal to the sum of
 *                 coefficients, this will keep the maximum result value within
 *                 full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterRow_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jobject pKernel, jint nMaskSize, jint nAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilterRow_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterRow_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, pKernel=%p, nMaskSize=%d, nAnchor=%d, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, pKernel, nMaskSize, nAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s* pKernel_native = NULL;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilterRow_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, pKernel_native, nMaskSize_native, nAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nMaskSize is primitive
    // nAnchor is primitive
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name 1D Window Sum
 *  1D mask Window Sum for 8 bit images.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned 1D (column) sum to 32f.
 *
 * Apply Column Window Summation filter over a 1D mask region around each
 * source pixel for 1-channel 8 bit/pixel input images with 32-bit floating point
 * output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring column pixel values in a mask region of the source image defined by
 * nMaskSize and nAnchor. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor Y offset of the kernel origin frame of reference w.r.t the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowColumn_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowColumn_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowColumn_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowColumn_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned 1D (row) sum to 32f.
 *
 * Apply Row Window Summation filter over a 1D mask region around each source
 * pixel for 1-channel 8-bit pixel input images with 32-bit floating point output.  
 * Result 32-bit floating point pixel is equal to the sum of the corresponding and
 * neighboring row pixel values in a mask region of the source image defined
 * by iKernelDim and iAnchorX. 
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param nMaskSize Length of the linear kernel array.
 * @param nAnchor X offset of the kernel origin frame of reference w.r.t the
 *        source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiSumWindowRow_18u32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint nMaskSize, jint nAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiSumWindowRow_8u32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nMaskSize is primitive
    // nAnchor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiSumWindowRow_8u32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, nMaskSize=%d, nAnchor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, nMaskSize, nAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp32f* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oROI_native;
    Npp32s nMaskSize_native;
    Npp32s nAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nMaskSize_native = (Npp32s)nMaskSize;
    nAnchor_native = (Npp32s)nAnchor;

    // Native function call
    NppStatus jniResult_native = nppiSumWindowRow_8u32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, nMaskSize_native, nAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // nMaskSize is primitive
    // nAnchor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Convolution (2D Masks)
 * General purpose 2D convolution filters.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned convolution filter.
 * 
 * Pixels under the mask are multiplied by the respective weights in the mask
 * and the results are summed. Before writing the result pixel the sum is scaled
 * back via division by nDivisor.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned convolution filter.
 * 
 * Pixels under the mask are multiplied by the respective weights in the mask
 * and the results are summed. Before writing the result pixel the sum is scaled
 * back via division by nDivisor.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pKernel Pointer to the start address of the kernel coefficient array.
 *        Coeffcients are expected to be stored in reverse order.
 * @param oKernelSize Width and Height of the rectangular kernel.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @param nDivisor The factor by which the convolved summation from the Filter
 *        operation should be divided.  If equal to the sum of coefficients,
 *        this will keep the maximum result value within full scale.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilter_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pKernel, jobject oKernelSize, jobject oAnchor, jint nDivisor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pKernel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pKernel' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oKernelSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oKernelSize' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilter_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDivisor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilter_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pKernel=%p, oKernelSize=%p, oAnchor=%p, nDivisor=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pKernel, oKernelSize, oAnchor, nDivisor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp32s* pKernel_native = NULL;
    NppiSize oKernelSize_native;
    NppiPoint oAnchor_native;
    Npp32s nDivisor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pKernel_pointerData = initPointerData(env, pKernel);
    if (pKernel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pKernel_native = (Npp32s*)pKernel_pointerData->getPointer(env);
    if (!initNative(env, oKernelSize, oKernelSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nDivisor_native = (Npp32s)nDivisor;

    // Native function call
    NppStatus jniResult_native = nppiFilter_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pKernel_native, oKernelSize_native, oAnchor_native, nDivisor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pKernel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oKernelSize is a read-only structure
    // oAnchor is a read-only structure
    // nDivisor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name 2D Linear Fixed Filters
 *  2D linear fixed filters for 8 bit images.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned box filter.
 *
 * Computes the average pixel values of the pixels under a rectangular mask.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned box filter.
 *
 * Computes the average pixel values of the pixels under a rectangular mask.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Avg operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference w.r.t
 *        the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterBox_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterBox_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterBox_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterBox_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Image Rank Filters
 *  Min, Median, and Max image filters.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned maximum filter.
 *
 * Result pixel value is the maximum of pixel values under the rectangular
 * mask region.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned maximum filter.
 *
 * Result pixel value is the maximum of pixel values under the rectangular
 * mask region.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMax_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMax_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMax_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMax_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned minimum filter.
 *
 * Result pixel value is the minimum of pixel values under the rectangular
 * mask region.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned minimum filter.
 *
 * Result pixel value is the minimum of pixel values under the rectangular
 * mask region.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param oMaskSize Width and Height of the neighborhood region for the local
 *        Max operation.
 * @param oAnchor X and Y offsets of the kernel origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiFilterMin_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiFilterMin_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiFilterMin_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiFilterMin_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
///@} image_filtering_functions
/**
 * <pre>
 * @defgroup image_morphological_operations Morphological Operations
 *
 * Morphological image operations.
 *
 *
 * </pre>
 */
///@{
/**
 * <pre>
 * 8-bit unsigned image dilation.
 * 
 * Dilation computes the output pixel as the maximum pixel value of the pixels
 * under the mask. Pixels who's corresponding mask values are zero to not 
 * participate in the maximum search.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image dilation.
 * 
 * Dilation computes the output pixel as the maximum pixel value of the pixels
 * under the mask. Pixels who's corresponding mask values are zero to not 
 * participate in the maximum search.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDilate_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiDilate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDilate_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDilate_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image erosion.
 * 
 * Erosion computes the output pixel as the minimum pixel value of the pixels
 * under the mask. Pixels who's corresponding mask values are zero to not 
 * participate in the maximum search.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image erosion.
 * 
 * Erosion computes the output pixel as the minimum pixel value of the pixels
 * under the mask. Pixels who's corresponding mask values are zero to not 
 * participate in the maximum search.
 *
 * @param pSrc  \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pMask Pointer to the start address of the mask array
 * @param oMaskSize Width and Height mask array.
 * @param oAnchor X and Y offsets of the mask origin frame of reference
 *        w.r.t the source pixel.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiErode_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pMask, jobject oMaskSize, jobject oAnchor)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pMask == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pMask' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oMaskSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oMaskSize' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oAnchor == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oAnchor' is null for nppiErode_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiErode_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pMask=%p, oMaskSize=%p, oAnchor=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pMask, oMaskSize, oAnchor);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    Npp32s nSrcStep_native;
    Npp8u* pDst_native = NULL;
    Npp32s nDstStep_native;
    NppiSize oSizeROI_native;
    Npp8u* pMask_native = NULL;
    NppiSize oMaskSize_native;
    NppiPoint oAnchor_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (Npp32s)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (Npp32s)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pMask_pointerData = initPointerData(env, pMask);
    if (pMask_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pMask_native = (Npp8u*)pMask_pointerData->getPointer(env);
    if (!initNative(env, oMaskSize, oMaskSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oAnchor, oAnchor_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiErode_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pMask_native, oMaskSize_native, oAnchor_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pMask_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oMaskSize is a read-only structure
    // oAnchor is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} image_morphological_operations
/**
 * <pre>
 * @defgroup image_linear_transforms Image Linear Transforms
 *
 * Linear image transformations.
 *
 *
 * </pre>
 */
///@{
/**
 * <pre>
 * 32-bit floating point complex to 32-bit floating point magnitude.
 * 
 * Converts complex-number pixel image to single channel image computing
 * the result pixels as the magnitude of the complex values.
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMagnitude_132fc32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMagnitude_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMagnitude_32fc32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMagnitude_32fc32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit floating point complex to 32-bit floating point squared magnitude.
 * 
 * Converts complex-number pixel image to single channel image computing
 * the result pixels as the squared magnitude of the complex values.
 * 
 * The squared magnitude is an itermediate result of magnitude computation and
 * can thus be computed faster than actual magnitude. If magnitudes are required
 * for sorting/comparing only, using this function instead of nppiMagnitude_32fc32f_C1R
 * can be a worthwhile performance optimization.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMagnitudeSqr_132fc32f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiMagnitudeSqr_32fc32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMagnitudeSqr_32fc32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp32fc* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32fc*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiMagnitudeSqr_32fc32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} image_linear_transforms
/**
 * <pre>
 * @defgroup image_compression Compression
 *
 * Image compression primitives.
 *
 * The JPEG standard defines a flow of level shift, DCT and quantization for
 * forward JPEG transform and inverse level shift, IDCT and de-quantization
 * for inverse JPEG transform. This group has the functions for both forward
 * and inverse functions. 
 * </pre>
 */
///@{
/**
 * <pre>
 * Apply quality factor to raw 8-bit quantization table.
 *
 * This is effectively and in-place method that modifies a given raw
 * quantization table based on a quality factor.
 * Note that this method is a host method and that the pointer to the
 * raw quantization table is a host pointer.
 *
 * @param hpQuantRawTable Raw quantization table.
 * @param nQualityFactor Quality factor for the table. Range is [1:100].
 * @return
 *  Error code:
 *      #NPP_NULL_POINTER_ERROR is returned if hpQuantRawTable is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantFwdRawTableInit_1JPEG_18uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jint nQualityFactor)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantFwdRawTableInit_JPEG_8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nQualityFactor is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantFwdRawTableInit_JPEG_8u(hpQuantRawTable=%p, nQualityFactor=%d)\n",
        hpQuantRawTable, nQualityFactor);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    int nQualityFactor_native = 0;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    nQualityFactor_native = (int)nQualityFactor;

    // Native function call
    NppStatus jniResult_native = nppiQuantFwdRawTableInit_JPEG_8u(hpQuantRawTable_native, nQualityFactor_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nQualityFactor is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes a quantization table for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R().
 *    The method creates a 16-bit version of the raw table and converts the 
 * data order from zigzag layout to original row-order layout since raw
 * quantization tables are typically stored in zigzag format.
 *
 * This method is a host method. It consumes and produces host data. I.e. the pointers
 * passed to this function must be host pointers. The resulting table needs to be
 * transferred to device memory in order to be used with nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R()
 * function.
 *
 * @param hpQuantRawTable Host pointer to raw quantization table as returned by 
 *      nppiQuantFwdRawTableInit_JPEG_8u(). The raw quantization table is assumed to be in
 *      zigzag order.
 * @param hpQuantFwdRawTable Forward quantization table for use with nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R().
 * @return
 *  Error code:
 *      #NPP_NULL_POINTER_ERROR pQuantRawTable is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantFwdTableInit_1JPEG_18u16uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jobject hpQuantFwdRawTable)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantFwdTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpQuantFwdRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantFwdRawTable' is null for nppiQuantFwdTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantFwdTableInit_JPEG_8u16u(hpQuantRawTable=%p, hpQuantFwdRawTable=%p)\n",
        hpQuantRawTable, hpQuantFwdRawTable);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    Npp16u* hpQuantFwdRawTable_native = NULL;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    PointerData *hpQuantFwdRawTable_pointerData = initPointerData(env, hpQuantFwdRawTable);
    if (hpQuantFwdRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantFwdRawTable_native = (Npp16u*)hpQuantFwdRawTable_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQuantFwdTableInit_JPEG_8u16u(hpQuantRawTable_native, hpQuantFwdRawTable_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpQuantFwdRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes a quantization table for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R().
 *      The nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R() method uses a quantization table
 * in a 16-bit format allowing for faster processing. In addition it converts the 
 * data order from zigzag layout to original row-order layout. Typically raw
 * quantization tables are stored in zigzag format.
 *
 * This method is a host method and consumes and produces host data. I.e. the pointers
 * passed to this function must be host pointers. The resulting table needs to be
 * transferred to device memory in order to be used with nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R()
 * function.
 *
 * @param hpQuantRawTable Raw quantization table.
 * @param hpQuantFwdRawTable Inverse quantization table.
 * @return
 *  #NPP_NULL_POINTER_ERROR pQuantRawTable or pQuantFwdRawTable is0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiQuantInvTableInit_1JPEG_18u16uNative(JNIEnv *env, jclass cls, jobject hpQuantRawTable, jobject hpQuantFwdRawTable)
{
    // Null-checks for non-primitive arguments
    if (hpQuantRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantRawTable' is null for nppiQuantInvTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (hpQuantFwdRawTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'hpQuantFwdRawTable' is null for nppiQuantInvTableInit_JPEG_8u16u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiQuantInvTableInit_JPEG_8u16u(hpQuantRawTable=%p, hpQuantFwdRawTable=%p)\n",
        hpQuantRawTable, hpQuantFwdRawTable);

    // Native variable declarations
    Npp8u* hpQuantRawTable_native = NULL;
    Npp16u* hpQuantFwdRawTable_native = NULL;

    // Obtain native variable values
    PointerData *hpQuantRawTable_pointerData = initPointerData(env, hpQuantRawTable);
    if (hpQuantRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantRawTable_native = (Npp8u*)hpQuantRawTable_pointerData->getPointer(env);
    PointerData *hpQuantFwdRawTable_pointerData = initPointerData(env, hpQuantFwdRawTable);
    if (hpQuantFwdRawTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    hpQuantFwdRawTable_native = (Npp16u*)hpQuantFwdRawTable_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiQuantInvTableInit_JPEG_8u16u(hpQuantRawTable_native, hpQuantFwdRawTable_native);

    // Write back native variable values
    if (!releasePointerData(env, hpQuantRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, hpQuantFwdRawTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Forward DCT, quantization and level shift part of the JPEG encoding.
 * Input is expected in 8x8 macro blocks and output is expected to be in 64x1
 * macro blocks.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pQuantFwdTable Forward quantization tables for JPEG encoding created
 *          using nppiQuantInvTableInit_JPEG_8u16u().
 * @param oSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - #NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - #NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantFwd8x8LS_1JPEG_18u16s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantFwdTable, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantFwdTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantFwdTable' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantFwdTable=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantFwdTable, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16s* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp16u* pQuantFwdTable_native = NULL;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantFwdTable_pointerData = initPointerData(env, pQuantFwdTable);
    if (pQuantFwdTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantFwdTable_native = (Npp16u*)pQuantFwdTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantFwd8x8LS_JPEG_8u16s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantFwdTable_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantFwdTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse DCT, de-quantization and level shift part of the JPEG decoding.
 * Input is expected in 64x1 macro blocks and output is expected to be in 8x8
 * macro blocks.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param pQuantInvTable Inverse quantization tables for JPEG decoding created
 *           using nppiQuantInvTableInit_JPEG_8u16u().
 * @param oSizeROI \ref roi_specification.
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR For negative input height/width or not a multiple of
 *           8 width/height.
 *         - #NPP_STEP_ERROR If input image width is not multiple of 8 or does not
 *           match ROI.
 *         - #NPP_NULL_POINTER_ERROR If the destination pointer is 0.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiDCTQuantInv8x8LS_1JPEG_116s8u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject pQuantInvTable, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (pQuantInvTable == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pQuantInvTable' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, pQuantInvTable=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, pQuantInvTable, oSizeROI);

    // Native variable declarations
    Npp16s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    Npp16u* pQuantInvTable_native = NULL;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    PointerData *pQuantInvTable_pointerData = initPointerData(env, pQuantInvTable);
    if (pQuantInvTable_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pQuantInvTable_native = (Npp16u*)pQuantInvTable_pointerData->getPointer(env);
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiDCTQuantInv8x8LS_JPEG_16s8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, pQuantInvTable_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    if (!releasePointerData(env, pQuantInvTable_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@} image_compression
/**
 * <pre>
 * @defgroup image_geometric_transforms Geometric Transforms
 *
 * Routines manipulating an image's geometry.
 *
 * </pre>
 */
///@{
/** @name Resize
 *  Resizes 8 bit images. Handles C1 and C4 images.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image
 * @param xFactor Factors by which x dimension is changed 
 * @param yFactor Factors by which y dimension is changed 
 * @param eInterpolation The type of eInterpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if
 *           srcROIRect has no intersection with the source image.
 *         - #NPP_RESIZE_NO_OPERATION_ERROR if either destination ROI width or
 *           height is less than 1 pixel.
 *         - #NPP_RESIZE_FACTOR_ERROR Indicates an error condition if either xFactor or
 *           yFactor is less than or equal to zero.
 *         - #NPP_INTERPOLATION_ERROR if eInterpolation has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble xFactor, jdouble yFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // xFactor is primitive
    // yFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, xFactor=%lf, yFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, xFactor, yFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double xFactor_native = 0.0;
    double yFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    xFactor_native = (double)xFactor;
    yFactor_native = (double)yFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, xFactor_native, yFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // xFactor is primitive
    // yFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image resize.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstROISize Size in pixels of the destination image
 * @param xFactor Factors by which x dimension is changed 
 * @param yFactor Factors by which y dimension is changed 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR indicates an error condition if
 *           srcROIRect has no intersection with the source image.
 *         - #NPP_RESIZE_NO_OPERATION_ERROR if either destination ROI width or
 *           height is less than 1 pixel.
 *         - #NPP_RESIZE_FACTOR_ERROR Indicates an error condition if either xFactor or
 *           yFactor is less than or equal to zero.
 *         - #NPP_INTERPOLATION_ERROR if eInterpolation has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiResize_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject dstROISize, jdouble xFactor, jdouble yFactor, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstROISize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstROISize' is null for nppiResize_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // xFactor is primitive
    // yFactor is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiResize_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, dstROISize=%p, xFactor=%lf, yFactor=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, dstROISize, xFactor, yFactor, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize dstROISize_native;
    double xFactor_native = 0.0;
    double yFactor_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstROISize, dstROISize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    xFactor_native = (double)xFactor;
    yFactor_native = (double)yFactor;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiResize_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, dstROISize_native, xFactor_native, yFactor_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstROISize is a read-only structure
    // xFactor is primitive
    // yFactor is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Rotate
 *  Rotates an image around the origin (0,0) and then shifts it.
 *
 * </pre>
 */
///@{
/**
 * <pre>
 * Compute shape of rotated image.
 * 
 * @param oSrcROI Region-of-interest of the source image.
 * @param aQuad Array of 2D points. These points are the locations of the corners
 *      of the rotated ROI. 
 * @param nAngle The rotation nAngle.
 * @param nShiftX Post-rotation shift in x-direction
 * @param nShiftY Post-rotation shift in y-direction
 * @return
 *  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetRotateQuadNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aQuad, jdouble nAngle, jdouble nShiftX, jdouble nShiftY)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetRotateQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aQuad' is null for nppiGetRotateQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetRotateQuad(oSrcROI=%p, aQuad=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf)\n",
        oSrcROI, aQuad, nAngle, nShiftX, nShiftY);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aQuad_native[4][2];
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aQuad, aQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;

    // Native function call
    NppStatus jniResult_native = nppiGetRotateQuad(oSrcROI_native, aQuad_native, nAngle_native, nShiftX_native, nShiftY_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aQuad_native, aQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Compute bounding-box of rotated image.
 * @param oSrcROI Region-of-interest of the source image.
 * @param aBoundingBox Two 2D points representing the bounding-box of the rotated image. All four points
 *      from nppiGetRotateQuad are contained inside the axis-aligned rectangle spanned by the the two
 *      points of this bounding box.
 * @param nAngle The rotation angle.
 * @param nShiftX Post-rotation shift in x-direction.
 * @param nShiftY Post-rotation shift in y-direction.
 *
 * @return
 *  \ref roi_error_codes.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetRotateBoundNative(JNIEnv *env, jclass cls, jobject oSrcROI, jobjectArray aBoundingBox, jdouble nAngle, jdouble nShiftX, jdouble nShiftY)
{
    // Null-checks for non-primitive arguments
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiGetRotateBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aBoundingBox == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aBoundingBox' is null for nppiGetRotateBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetRotateBound(oSrcROI=%p, aBoundingBox=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf)\n",
        oSrcROI, aBoundingBox, nAngle, nShiftX, nShiftY);

    // Native variable declarations
    NppiRect oSrcROI_native;
    double aBoundingBox_native[2][2];
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;

    // Obtain native variable values
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, aBoundingBox, aBoundingBox_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;

    // Native function call
    NppStatus jniResult_native = nppiGetRotateBound(oSrcROI_native, aBoundingBox_native, nAngle_native, nShiftX_native, nShiftY_native);

    // Write back native variable values
    // oSrcROI is a read-only structure
    if (!releaseNative(env, aBoundingBox_native, aBoundingBox)) return JNPP_STATUS_INTERNAL_ERROR;
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image rotate ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image rotate ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image rotate.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image rotate ignoring alpha channel.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param oSrcSize Size in pixels of the source image
 * @param oSrcROI Region of interest in the source image.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oDstROI Region of interest in the destination image.
 * @param nAngle The angle of rotation in degrees.
 * @param nShiftX Shift along horizontal axis 
 * @param nShiftY Shift along vertical axis 
 * @param eInterpolation The type of interpolation to perform resampling
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes, \ref rotate_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRotate_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject oSrcROI, jobject pDst, jint nDstStep, jobject oDstROI, jdouble nAngle, jdouble nShiftX, jdouble nShiftY, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (oSrcROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcROI' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oDstROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oDstROI' is null for nppiRotate_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRotate_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, oSrcROI=%p, pDst=%p, nDstStep=%d, oDstROI=%p, nAngle=%lf, nShiftX=%lf, nShiftY=%lf, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, oSrcROI, pDst, nDstStep, oDstROI, nAngle, nShiftX, nShiftY, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect oSrcROI_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect oDstROI_native;
    double nAngle_native = 0.0;
    double nShiftX_native = 0.0;
    double nShiftY_native = 0.0;
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, oSrcROI, oSrcROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oDstROI, oDstROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nAngle_native = (double)nAngle;
    nShiftX_native = (double)nShiftX;
    nShiftY_native = (double)nShiftY;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiRotate_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, oSrcROI_native, pDst_native, nDstStep_native, oDstROI_native, nAngle_native, nShiftX_native, nShiftY_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // oSrcROI is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oDstROI is a read-only structure
    // nAngle is primitive
    // nShiftX is primitive
    // nShiftY is primitive
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Mirror
 *  Mirrors images horizontally, vertically and diagonally.
 */
///@{
/**
 * <pre>
 * 8-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 16-bit unsigned image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_16u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_16u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32s_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32s_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 32-bit float image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 32-bit float image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image mirror.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_C4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_C4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 32-bit float image mirror not affecting alpha.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep Distance in bytes between starts of consecutive lines of the
 *        destination image.
 * @param oROI \ref roi_specification.
 * @param flip Specifies the axis about which the image is to be mirrored.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_MIRROR_FLIP_ERR if flip has an illegal value.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiMirror_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oROI, jint flip)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oROI' is null for nppiMirror_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // flip is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiMirror_32f_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oROI=%p, flip=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oROI, flip);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oROI_native;
    NppiAxis flip_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oROI, oROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    flip_native = (NppiAxis)flip;

    // Native function call
    NppStatus jniResult_native = nppiMirror_32f_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oROI_native, flip_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oROI is a read-only structure
    // flip is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Affine warping, affine transform calculation
 * Affine warping of an image is the transform of image pixel positions,
 * defined by the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * C = \left[ \matrix{C_{00} & C_{01} & C_{02} \cr C_{10} & C_{11} & C_{12} } \right]
 * \f]
 * That is, any pixel with coordinates 
 * \f$(X_{new},Y_{new})\f$ in the transformed image is sourced from 
 * coordinates \f$(x,y)\f$ in the original image. The mapping \f$C\f$ is completely
 * specified by 6 values
 * \f$C_{ij}, i=\overline{0,1}, j=\overline{0,2}\f$.
 * The transform maps parallel lines to parallel
 * lines and preserves ratios of distances of points to lines.
 * Implementation specific properties are
 * discussed in each function's documentation.
 * </pre>
 */
///@{
/**
 * <pre>
 * Calculates affine transform coefficients given source rectangular ROI and
 * its destination quadrangle projection
 *
 * @param srcRoi Source ROI
 * @param quad Destination quadrangle
 * @param coeffs Affine transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_AFFINE_QUAD_INCORRECT_WARNING Indicates a warning when quad
 *           does not conform to the transform properties. Fourth vertex is
 *           ignored, internally computed coordinates are used instead
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineTransformNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray quad, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetAffineTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineTransform(srcRoi=%p, quad=%p, coeffs=%p)\n",
        srcRoi, quad, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double quad_native[4][2];
    double coeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineTransform(srcRoi_native, quad_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates affine transform projection of given source rectangular ROI
 *
 * @param srcRoi Source ROI
 * @param quad Destination quadrangle
 * @param coeffs Affine transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineQuadNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray quad, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetAffineQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineQuad(srcRoi=%p, quad=%p, coeffs=%p)\n",
        srcRoi, quad, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double quad_native[4][2];
    double coeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineQuad(srcRoi_native, quad_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates bounding box of the affine transform projection of the given
 * source rectangular ROI
 *
 * @param srcRoi Source ROI
 * @param bound Bounding box of the transformed source ROI
 * @param coeffs Affine transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetAffineBoundNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray bound, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (bound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'bound' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetAffineBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetAffineBound(srcRoi=%p, bound=%p, coeffs=%p)\n",
        srcRoi, bound, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double bound_native[2][2];
    double coeffs_native[2][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, bound, bound_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetAffineBound(srcRoi_native, bound_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, bound_native, bound)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, single channel). This
 * function operates using given transform coefficients that can be obtained
 * by using nppiGetAffineTransform function or set explicitly. The function
 * operates on source and destination regions of interest. The affine warp
 * function transforms the source image pixel coordinates \f$(x,y)\f$ according
 * to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with 
 * destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated memory. In case
 * when the conditions above are not satisfied, the function may decrease in
 * speed slightly and will return NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           interpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpAffine_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpAffine_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpAffine_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpAffine_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpAffine_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, single channel).
 * This function operates using given transform coefficients that can be
 * obtained by using nppiGetAffineTransform function or set explicitly. Thus
 * there is no need to invert coefficients in your application before calling
 * WarpAffineBack. The function operates on source and destination regions of
 * interest.
 * The affine warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes aligned. This is
 * always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected image are separated from
 * the ROI by at least 63 bytes from each side. However, this requires the
 * whole ROI to be part of allocated memory. In case when the conditions above
 * are not satisfied, the function may decrease in speed slightly and will
 * return NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           interpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpAffineBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpAffineBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpAffineBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpAffineBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpAffineBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, single channel). This
 * function performs affine warping of a the specified quadrangle in the
 * source image to the specified quadrangle in the destination image. The
 * function nppiWarpAffineQuad uses the same formulas for pixel mapping as in
 * nppiWarpAffine function. The transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * eInterpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant, but does not
 * perform memory access checks and requires the destination ROI to be 64 bytes
 * aligned. Hence any destination ROI is chunked into 3 vertical stripes: the
 * first and the third are processed by accurate kernels and the central one is
 * processed by the fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes aligned. This is
 * always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected image are separated from
 * the ROI by at least 63 bytes from each side. However, this requires the
 * whole ROI to be part of allocated memory. In case when the conditions above
 * are not satisfied, the function may decrease in speed slightly and will
 * return NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 *         ignored, internally computed coordinates are used instead
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpAffineQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpAffineQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpAffineQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpAffineQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpAffineQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, single channel). This
 * function operates using given transform coefficients that can be obtained by
 * using nppiGetAffineTransform function or set explicitly. The function
 * operates on source and destination regions of interest. The affine warp
 * function transforms the source image pixel coordinates \f$(x,y)\f$
 * according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant, but does not
 * perform memory access checks and requires the destination ROI to be 64 bytes
 * aligned. Hence any destination ROI is chunked into 3 vertical stripes: the
 * first and the third are processed by accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes aligned. This is
 * always true if the values <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt>  are multiples of
 * 64. Another rule of thumb is to specify destination ROI in such way that
 * left and right sides of the projected image are separated from the ROI by at
 * least 63 bytes from each side. However, this requires the whole ROI to be
 * part of allocated memory. In case when the conditions above are not
 * satisfied, the function may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, three channels)
 * @see nppiWarpAffine_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four channels)
 * @see nppiWarpAffine_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four channels RGBA)
 * @see nppiWarpAffine_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, three planes)
 * @see nppiWarpAffine_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four planes)
 * @see nppiWarpAffine_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, single 
 * channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly.
 * Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and
 * destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function may
 * decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, three channels)
 * @see nppiWarpAffineBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, four channels)
 * @see nppiWarpAffineBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, four channels
 * RGBA)
 * @see nppiWarpAffineBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, three planes)
 * @see nppiWarpAffineBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (16bit unsigned integer, four planes)
 * @see nppiWarpAffineBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, single channel). This
 * function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but doesn't perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in 
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, three channels). 
 * @see nppiWarpAffineQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four channels). 
 * @see nppiWarpAffineQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four channels RGBA). 
 * @see nppiWarpAffineQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, three planes). 
 * @see nppiWarpAffineQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (16bit unsigned integer, four planes). 
 * @see nppiWarpAffineQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, single channel). This function
 * operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly.
 * The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, three channels).
 * @see nppiWarpAffine_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four channels).
 * @see nppiWarpAffine_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpAffine_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, three planes).
 * @see nppiWarpAffine_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four planes).
 * @see nppiWarpAffine_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, single channel). This
 * function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly.
 * Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and
 * destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, three channels).
 * @see nppiWarpAffineBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, four channels).
 * @see nppiWarpAffineBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpAffineBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, three planes).
 * @see nppiWarpAffineBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit float, four planes).
 * @see nppiWarpAffineBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, single channel). This function
 * performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, three channels).
 * @see nppiWarpAffineQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four channels).
 * @see nppiWarpAffineQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpAffineQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, three planes).
 * @see nppiWarpAffineQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit float, four planes).
 * @see nppiWarpAffineQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, single channel). This
 * function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly.
 * The function operates on source and destination regions 
 * of interest. The affine warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = C_{00} * x + C_{01} * y + C_{02} \qquad
 * Y_{new} = C_{10} * x + C_{11} * y + C_{12} \qquad
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, three channels).
 * @see nppiWarpAffine_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint interpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // interpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, interpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, interpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int interpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    interpolation_native = (int)interpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, interpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // interpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four channels).
 * @see nppiWarpAffine_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpAffine_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, three planes).
 * @see nppiWarpAffine_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four planes).
 * @see nppiWarpAffine_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffine_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffine_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffine_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffine_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, single
 * channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetAffineTransform function or set explicitly.
 * Thus there is no need to invert coefficients in your application 
 * before calling WarpAffineBack. The function operates on source and
 * destination regions of interest.
 * The affine warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * C_{00} * X_{new} + C_{01} * Y_{new} + C_{02} = x  \qquad
 * C_{10} * X_{new} + C_{11} * Y_{new} + C_{12} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetAffineQuad and nppiGetAffineBound can help with
 * destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Affine transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, three channels).
 * @see nppiWarpAffineBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, four channels).
 * @see nppiWarpAffineBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpAffineBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, three planes).
 * @see nppiWarpAffineBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse affine transform of an image (32bit signed integer, four planes).
 * @see nppiWarpAffineBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineBack_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpAffineBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineBack_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[2][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineBack_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, single channel). This
 * function performs affine warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpAffineQuad uses the same
 * formulas for pixel mapping as in nppiWarpAffine function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 *           misalignment
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, three channels).
 * @see nppiWarpAffineQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four channels).
 * @see nppiWarpAffineQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpAffineQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, three planes).
 * @see nppiWarpAffineQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Affine transform of an image (32bit signed integer, four planes).
 * @see nppiWarpAffineQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpAffineQuad_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpAffineQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpAffineQuad_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpAffineQuad_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/**
 * <pre>
 * @name Perspective warping, perspective transform calculation
 *  Perspective warping of an image is the transform of image pixel positions,
 * defined by the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * C = \left[ \matrix{C_{00} & C_{01} & C_{02} \cr C_{10} & C_{11} & C_{12} \cr C_{20} & C_{21} & C_{22} } \right]
 * \f]
 * That is, any pixel of the transformed image with coordinates 
 * \f$(X_{new},Y_{new})\f$ has a preimage with 
 * coordinates \f$(x,y)\f$. The mapping \f$C\f$ is fully defined by 8 values
 * \f$C_{ij}, (i,j)=\overline{0,2}\f$, except of \f$C_{22}\f$, which is a
 * normalizer.
 * The transform has a property of mapping any convex quadrangle to a convex
 * quadrangle, which is used in a group of functions nppiWarpPerspectiveQuad.
 * The NPPI implementation of perspective transform has some issues which are
 * discussed in each function's documentation.
 * </pre>
 */
///@{
/**
 * <pre>
 * Calculates perspective transform coefficients given source rectangular ROI
 * and its destination quadrangle projection
 *
 * @param srcRoi Source ROI
 * @param quad Destination quadrangle
 * @param coeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveTransformNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray quad, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetPerspectiveTransform");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveTransform(srcRoi=%p, quad=%p, coeffs=%p)\n",
        srcRoi, quad, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double quad_native[4][2];
    double coeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveTransform(srcRoi_native, quad_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates perspective transform projection of given source rectangular
 * ROI
 *
 * @param srcRoi Source ROI
 * @param quad Destination quadrangle
 * @param coeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveQuadNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray quad, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (quad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'quad' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetPerspectiveQuad");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveQuad(srcRoi=%p, quad=%p, coeffs=%p)\n",
        srcRoi, quad, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double quad_native[4][2];
    double coeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, quad, quad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveQuad(srcRoi_native, quad_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, quad_native, quad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates bounding box of the perspective transform projection of the
 * given source rectangular ROI
 *
 * @param srcRoi Source ROI
 * @param bound Bounding box of the transformed source ROI
 * @param coeffs Perspective transform coefficients
 * @return
 *  Error codes:
 *         - #NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *           has zero or negative value
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGetPerspectiveBoundNative(JNIEnv *env, jclass cls, jobject srcRoi, jobjectArray bound, jobjectArray coeffs)
{
    // Null-checks for non-primitive arguments
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (bound == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'bound' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiGetPerspectiveBound");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGetPerspectiveBound(srcRoi=%p, bound=%p, coeffs=%p)\n",
        srcRoi, bound, coeffs);

    // Native variable declarations
    NppiRect srcRoi_native;
    double bound_native[2][2];
    double coeffs_native[3][3];

    // Obtain native variable values
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, bound, bound_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiGetPerspectiveBound(srcRoi_native, bound_native, coeffs_native);

    // Write back native variable values
    // srcRoi is a read-only structure
    if (!releaseNative(env, bound_native, bound)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, single channel).
 * This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image
 * pixel coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpPerspective_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpPerspective_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpPerspective_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpPerspective_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpPerspective_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, single
 * channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and
 * destination regions of interest.
 * The perspective warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI 
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI
 * in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN, NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpPerspectiveBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpPerspectiveBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpPerspectiveBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpPerspectiveBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpPerspectiveBack_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, single channel).
 * This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The
 * transform coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection
 * of destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI
 * in such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, three channels).
 * @see nppiWarpPerspectiveQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four channels).
 * @see nppiWarpPerspectiveQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four channels RGBA).
 * @see nppiWarpPerspectiveQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, three planes).
 * @see nppiWarpPerspectiveQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (8bit unsigned integer, four planes).
 * @see nppiWarpPerspectiveQuad_8u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_18u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_8u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_8u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp8u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp8u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_8u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, single channel).
 * This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The
 * fast method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs  Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, three channels)
 * @see nppiWarpPerspective_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four channels)
 * @see nppiWarpPerspective_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four channels RGBA)
 * @see nppiWarpPerspective_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, three planes)
 * @see nppiWarpPerspective_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four planes)
 * @see nppiWarpPerspective_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, single
 * channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and
 * destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI
 * to be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, three channels)
 * @see nppiWarpPerspectiveBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, four channels)
 * @see nppiWarpPerspectiveBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, four channels RGBA)
 * @see nppiWarpPerspectiveBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, three planes)
 * @see nppiWarpPerspectiveBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (16bit unsigned integer, four planes)
 * @see nppiWarpPerspectiveBack_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, single channel).
 * This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * NPPI specific recommendation: 
 * The function operates using 2 types of kernels: fast and accurate. The fast
 * method is about 4 times faster than its accurate variant,
 * but does not perform memory access checks and requires the destination ROI to
 * be 64 bytes aligned. Hence any destination ROI is 
 * chunked into 3 vertical stripes: the first and the third are processed by
 * accurate kernels and the central one is processed by the
 * fast one.
 * In order to get the maximum available speed of execution, the projection of
 * destination ROI onto image addresses must be 64 bytes
 * aligned. This is always true if the values 
 * <tt>(int)((void *)(pDst + dstRoi.x))</tt> and 
 * <tt>(int)((void *)(pDst + dstRoi.x + dstRoi.width))</tt> 
 * are multiples of 64. Another rule of thumb is to specify destination ROI in
 * such way that left and right sides of the projected 
 * image are separated from the ROI by at least 63 bytes from each side.
 * However, this requires the whole ROI to be part of allocated 
 * memory. In case when the conditions above are not satisfied, the function
 * may decrease in speed slightly and will return 
 * NPP_MISALIGNED_DST_ROI_WARNING warning.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, three channels). 
 * @see nppiWarpPerspectiveQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four channels). 
 * @see nppiWarpPerspectiveQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four channels RGBA). 
 * @see nppiWarpPerspectiveQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp16u* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp16u*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp16u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, three planes). 
 * @see nppiWarpPerspectiveQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (16bit unsigned integer, four planes). 
 * @see nppiWarpPerspectiveQuad_16u_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_116u_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_16u_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_16u_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp16u* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp16u* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_16u_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, single channel). This
 * function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, three channels).
 * @see nppiWarpPerspective_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four channels).
 * @see nppiWarpPerspective_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpPerspective_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, three planes).
 * @see nppiWarpPerspective_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four planes).
 * @see nppiWarpPerspective_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, single channel).
 * This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. Thus there is no need to invert coefficients in your application 
 * before calling WarpPerspectiveBack. The function operates on source and
 * destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, three channels).
 * @see nppiWarpPerspectiveBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, four channels).
 * @see nppiWarpPerspectiveBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpPerspectiveBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, three planes).
 * @see nppiWarpPerspectiveBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit float, four planes).
 * @see nppiWarpPerspectiveBack_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, single channel).
 * This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, three channels).
 * @see nppiWarpPerspectiveQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four channels).
 * @see nppiWarpPerspectiveQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four channels RGBA).
 * @see nppiWarpPerspectiveQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32f* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32f*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32f*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, three planes).
 * @see nppiWarpPerspectiveQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit float, four planes).
 * @see nppiWarpPerspectiveQuad_32f_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132f_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32f_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32f_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32f* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32f* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32f_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, single channel).
 * This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. The function operates on source and destination regions 
 * of interest. The perspective warp function transforms the source image pixel
 * coordinates \f$(x,y)\f$ according to the following formulas:
 * \f[
 * X_{new} = \frac{C_{00} * x + C_{01} * y + C_{02}}{C_{20} * x + C_{21} * y + C_{22}} \qquad
 * Y_{new} = \frac{C_{10} * x + C_{11} * y + C_{12}}{C_{20} * x + C_{21} * y + C_{22}}
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs  Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, three channels).
 * @see nppiWarpPerspective_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four channels).
 * @see nppiWarpPerspective_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpPerspective_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, three planes).
 * @see nppiWarpPerspective_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four planes).
 * @see nppiWarpPerspective_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspective_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspective_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspective_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspective_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, single
 * channel). This function operates using given transform coefficients that 
 * can be obtained by using nppiGetPerspectiveTransform function or set
 * explicitly. Thus there is no need to invert coefficients in your
 * application 
 * before calling WarpPerspectiveBack. The function operates on source
 * and destination regions of interest.
 * The perspective warp function transforms the source image pixel coordinates
 * \f$(x,y)\f$ according to the following formulas:
 * \f[
 * \frac{C_{00} * X_{new} + C_{01} * Y_{new} + C_{02}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = x  \qquad
 * \frac{C_{10} * X_{new} + C_{11} * Y_{new} + C_{12}}{C_{20} * X_{new} + C_{21} * Y_{new} + C_{22}} = y
 * \f]
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * The functions nppiGetPerspectiveQuad and nppiGetPerspectiveBound can help
 * with destination ROI specification.
 *
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param coeffs Perspective transform coefficients
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, three channels).
 * @see nppiWarpPerspectiveBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, four channels).
 * @see nppiWarpPerspectiveBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpPerspectiveBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, three planes).
 * @see nppiWarpPerspectiveBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Inverse perspective transform of an image (32bit signed integer, four planes).
 * @see nppiWarpPerspectiveBack_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveBack_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray coeffs, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (coeffs == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'coeffs' is null for nppiWarpPerspectiveBack_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveBack_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, pDst=%p, nDstStep=%d, dstRoi=%p, coeffs=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, pDst, nDstStep, dstRoi, coeffs, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double coeffs_native[3][3];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, coeffs, coeffs_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveBack_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, pDst_native, nDstStep_native, dstRoi_native, coeffs_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, coeffs_native, coeffs)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, single channel).
 * This function performs perspective warping of a the specified
 * quadrangle in the source image to the specified quadrangle in the
 * destination image. The function nppiWarpPerspectiveQuad uses the same
 * formulas for pixel mapping as in nppiWarpPerspective function. The transform
 * coefficients are computed internally.
 * The transformed part of the source image is resampled using the specified
 * interpolation method and written to the destination ROI.
 * 
 * @param pSrc \ref source_image_pointer.
 * @param oSrcSize Size of source image in pixels
 * @param nSrcStep \ref source_image_line_step.
 * @param srcRoi Source ROI
 * @param srcQuad Source quadrangle
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param dstRoi Destination ROI
 * @param dstQuad Destination quadrangle
 * @param eInterpolation Interpolation mode: can be NPPI_INTER_NN,
 *        NPPI_INTER_LINEAR or NPPI_INTER_CUBIC
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *         - #NPP_RECT_ERROR Indicates an error condition if width or height of
 *           the intersection of the srcRoi and source image is less than or
 *           equal to 1
 *         - #NPP_WRONG_INTERSECTION_ROI_ERROR Indicates an error condition if
 *           srcRoi has no intersection with the source image
 *         - #NPP_INTERPOLATION_ERROR Indicates an error condition if
 *           eInterpolation has an illegal value
 *         - #NPP_COEFF_ERROR Indicates an error condition if coefficient values
 *           are invalid
 *         - #NPP_WRONG_INTERSECTION_QUAD_WARNING Indicates a warning that no
 *           operation is performed if the transformed source ROI has no
 *           intersection with the destination ROI
 *         - #NPP_MISALIGNED_DST_ROI_WARNING Indicates a warning that the speed
 *           of primitive execution was reduced due to destination ROI
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C1R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C1R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, three channels).
 * @see nppiWarpPerspectiveQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four channels).
 * @see nppiWarpPerspectiveQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1C4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_C4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_C4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_C4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four channels RGBA).
 * @see nppiWarpPerspectiveQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobject pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_AC4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    Npp32s* pSrc_native = NULL;
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp32s*)pSrc_pointerData->getPointer(env);
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp32s*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_AC4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, three planes).
 * @see nppiWarpPerspectiveQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_P3R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[3];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native[3];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_P3R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Perspective transform of an image (32bit signed integer, four planes).
 * @see nppiWarpPerspectiveQuad_32s_C1R
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiWarpPerspectiveQuad_132s_1P4RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jobject oSrcSize, jint nSrcStep, jobject srcRoi, jobjectArray srcQuad, jobjectArray pDst, jint nDstStep, jobject dstRoi, jobjectArray dstQuad, jint eInterpolation)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSrcSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSrcSize' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (srcRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcRoi' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (srcQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'srcQuad' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (dstRoi == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstRoi' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (dstQuad == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'dstQuad' is null for nppiWarpPerspectiveQuad_32s_P4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // eInterpolation is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiWarpPerspectiveQuad_32s_P4R(pSrc=%p, oSrcSize=%p, nSrcStep=%d, srcRoi=%p, srcQuad=%p, pDst=%p, nDstStep=%d, dstRoi=%p, dstQuad=%p, eInterpolation=%d)\n",
        pSrc, oSrcSize, nSrcStep, srcRoi, srcQuad, pDst, nDstStep, dstRoi, dstQuad, eInterpolation);

    // Native variable declarations
    const Npp32s* pSrc_native[4];
    NppiSize oSrcSize_native;
    int nSrcStep_native = 0;
    NppiRect srcRoi_native;
    double srcQuad_native[4][2];
    Npp32s* pDst_native[4];
    int nDstStep_native = 0;
    NppiRect dstRoi_native;
    double dstQuad_native[4][2];
    int eInterpolation_native = 0;

    // Obtain native variable values
    if (!initNative(env, pSrc, pSrc_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSrcSize, oSrcSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    nSrcStep_native = (int)nSrcStep;
    if (!initNative(env, srcRoi, srcRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, srcQuad, srcQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pDst, pDst_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, dstRoi, dstRoi_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, dstQuad, dstQuad_native)) return JNPP_STATUS_INTERNAL_ERROR;
    eInterpolation_native = (int)eInterpolation;

    // Native function call
    NppStatus jniResult_native = nppiWarpPerspectiveQuad_32s_P4R(pSrc_native, oSrcSize_native, nSrcStep_native, srcRoi_native, srcQuad_native, pDst_native, nDstStep_native, dstRoi_native, dstQuad_native, eInterpolation_native);

    // Write back native variable values
    // pSrc is an array of native pointers
    // oSrcSize is a read-only structure
    // nSrcStep is primitive
    // srcRoi is a read-only structure
    if (!releaseNative(env, srcQuad_native, srcQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // pDst is an array of native pointers
    // nDstStep is primitive
    // dstRoi is a read-only structure
    if (!releaseNative(env, dstQuad_native, dstQuad)) return JNPP_STATUS_INTERNAL_ERROR;
    // eInterpolation is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
///@} image_geometry_transforms
/**
 * <pre>
 * @defgroup image_color_conversion Color Conversion
 *
 * Image color space and sample conversion operations.
 *
 * </pre>
 */
///@{
/** @name RGBToYCbCr 
 *  RGB to YCbCr color conversion.
 */
///@{
/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to packed YCbCr color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned RGB to 2 channel chroma packed YCbCr422 color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr422_18u_1C3C2RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr422_8u_C3C2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr422_8u_C3C2R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr422_8u_C3C2R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned packed RGB to planar YCbCr420 color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr420_18u_1C3P3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr420_8u_C3P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr420_8u_C3P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u** pDst_native;
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr420_8u_C3P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel planar 8-bit unsigned RGB to YCbCr color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native = 0;
    Npp8u** pDst_native;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned RGB to YCbCr color conversion, ignoring Alpha.
 * 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiRGBToYCbCr_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiRGBToYCbCr_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiRGBToYCbCr_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiRGBToYCbCr_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name YCbCrToRGB 
 *  YCbCr to RGB color conversion.
 */
///@{
/**
 * <pre>
 * 3 channel 8-bit unsigned packed YCbCr to RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr to RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native = 0;
    Npp8u** pDst_native;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned packed YCbCr to RGB color conversion, not affecting Alpha.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCrToRGB_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCrToRGB_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCrToRGB_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCrToRGB_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 2 channel 8-bit unsigned YCbCr422 to 3 channel packed RGB color conversion.
 * images.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToRGB_18u_1C2C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToRGB_8u_C2C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToRGB_8u_C2C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToRGB_8u_C2C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr420 to packed RGB color conversion.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToRGB_18u_1P3C3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nSrcStep' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToRGB_8u_P3C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToRGB_8u_P3C3R(pSrc=%p, nSrcStep=%p, pDst=%p, nDstStep=%d, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native[3];
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nSrcStep, nSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToRGB_8u_P3C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nSrcStep_native, nSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Sample Pattern Conversion. 
 */
///@{
/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr:422 to YCbCr:420 resampling.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr420_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray nSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nSrcStep' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr420_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr420_8u_P3R(pSrc=%p, nSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native[3];
    Npp8u** pDst_native;
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nSrcStep, nSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr420_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nSrcStep_native, nSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr:422 to YCbCr:411 resampling.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr422ToYCbCr411_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray nSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nSrcStep' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr422ToYCbCr411_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr422ToYCbCr411_8u_P3R(pSrc=%p, nSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native[3];
    Npp8u** pDst_native;
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nSrcStep, nSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr422ToYCbCr411_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nSrcStep_native, nSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr:420 to YCbCr:422 resampling.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr422_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray nSrcStep, jobjectArray pDst, jintArray nDstStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nSrcStep' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nDstStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nDstStep' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr422_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr422_8u_P3R(pSrc=%p, nSrcStep=%p, pDst=%p, nDstStep=%p, oSizeROI=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native[3];
    Npp8u** pDst_native;
    int nDstStep_native[3];
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    if (!initNative(env, nSrcStep, nSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    if (!initNative(env, nDstStep, nDstStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr422_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nSrcStep_native, nSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, nDstStep_native, nDstStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned planar YCbCr:420 to YCbCr:411 resampling.
 *
 * @param pSrc Array of pointers to the source image planes.
 * @param aSrcStep Array with distances in bytes between starts of consecutive
 *        lines of the source image planes.
 * @param pDstY \ref destination_image_pointer. Y-channel.
 * @param nDstYStep \ref destination_image_line_step. Y-channel.
 * @param pDstCbCr \ref destination_image_pointer. CbCr image.
 * @param nDstCbCrStep \ref destination_image_line_step. CbCr image.
 * @param oSizeROI \ref roi_specification.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiYCbCr420ToYCbCr411_18u_1P3P2RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jintArray aSrcStep, jobject pDstY, jint nDstYStep, jobject pDstCbCr, jint nDstCbCrStep, jobject oSizeROI)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (aSrcStep == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'aSrcStep' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDstY == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstY' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstYStep is primitive
    if (pDstCbCr == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDstCbCr' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstCbCrStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiYCbCr420ToYCbCr411_8u_P3P2R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiYCbCr420ToYCbCr411_8u_P3P2R(pSrc=%p, aSrcStep=%p, pDstY=%p, nDstYStep=%d, pDstCbCr=%p, nDstCbCrStep=%d, oSizeROI=%p)\n",
        pSrc, aSrcStep, pDstY, nDstYStep, pDstCbCr, nDstCbCrStep, oSizeROI);

    // Native variable declarations
    Npp8u** pSrc_native;
    int aSrcStep_native[3];
    Npp8u* pDstY_native = NULL;
    int nDstYStep_native = 0;
    Npp8u* pDstCbCr_native = NULL;
    int nDstCbCrStep_native = 0;
    NppiSize oSizeROI_native;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    if (!initNative(env, aSrcStep, aSrcStep_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pDstY_pointerData = initPointerData(env, pDstY);
    if (pDstY_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstY_native = (Npp8u*)pDstY_pointerData->getPointer(env);
    nDstYStep_native = (int)nDstYStep;
    PointerData *pDstCbCr_pointerData = initPointerData(env, pDstCbCr);
    if (pDstCbCr_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDstCbCr_native = (Npp8u*)pDstCbCr_pointerData->getPointer(env);
    nDstCbCrStep_native = (int)nDstCbCrStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiYCbCr420ToYCbCr411_8u_P3P2R(pSrc_native, aSrcStep_native, pDstY_native, nDstYStep_native, pDstCbCr_native, nDstCbCrStep_native, oSizeROI_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releaseNative(env, aSrcStep_native, aSrcStep, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDstY_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstYStep is primitive
    if (!releasePointerData(env, pDstCbCr_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstCbCrStep is primitive
    // oSizeROI is a read-only structure

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

///@}
/** @name Color Processing
 *  Color manipuliation functions.
 */
///@{
/**
 * <pre>
 * 3 channel 8-bit unsigned color twist.
 * 
 * An input color twist matrix with floating-point pixel values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param twist The color twist matrix with floating-point pixel values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray twist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (twist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'twist' is null for nppiColorTwist32f_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, twist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, twist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f twist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, twist, twist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, twist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, twist_native, twist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel planar 8-bit unsigned color twist.
 *
 * An input color twist matrix with floating-point pixel values is applied
 * within ROI.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param twist The color twist matrix with floating-point pixel values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1P3RNative(JNIEnv *env, jclass cls, jobjectArray pSrc, jint nSrcStep, jobjectArray pDst, jint nDstStep, jobject oSizeROI, jobjectArray twist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (twist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'twist' is null for nppiColorTwist32f_8u_P3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_P3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, twist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, twist);

    // Native variable declarations
    Npp8u** pSrc_native;
    int nSrcStep_native = 0;
    Npp8u** pDst_native;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f twist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u**)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u**)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, twist, twist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_P3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, twist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, twist_native, twist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned color twist, not affecting Alpha.
 *
 * An input color twist matrix with floating-point pixel values is applied with
 * in ROI.
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param twist The color twist matrix with floating-point pixel values.
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiColorTwist32f_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray twist)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (twist == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'twist' is null for nppiColorTwist32f_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiColorTwist32f_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, twist=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, twist);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32f twist_native[3][4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, twist, twist_native)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiColorTwist32f_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, twist_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releaseNative(env, twist_native, twist)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 8-bit unsigned look-up-table color conversion.
 *
 * The LUT is derived from a set of user defined mapping points through linear
 * interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Pointer to an array of user defined OUTPUT values
 * @param pLevels Pointer to an array of user defined INPUT values
 * @param nLevels Number of user defined input/output mapping points (levels)
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - #NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than
 *         2.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C1RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobject pValues, jobject pLevels, jint nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C1R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLevels is primitive

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C1R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%d)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    Npp32s* pValues_native = NULL;
    Npp32s* pLevels_native = NULL;
    int nLevels_native = 0;

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pValues_pointerData = initPointerData(env, pValues);
    if (pValues_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pValues_native = (Npp32s*)pValues_pointerData->getPointer(env);
    PointerData *pLevels_pointerData = initPointerData(env, pLevels);
    if (pLevels_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLevels_native = (Npp32s*)pLevels_pointerData->getPointer(env);
    nLevels_native = (int)nLevels;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C1R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    if (!releasePointerData(env, pValues_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLevels_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLevels is primitive

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 3 channel 8-bit unsigned look-up-table color conversion.
 * 
 * The LUT is derived from a set of user defined mapping points through linear
 * interpolation. 
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Double pointer to an [3] of arrays of user defined OUTPUT
 *        values per CHANNEL
 * @param pLevels Double pointer to an [3] of arrays of user defined INPUT
 *        values per CHANNEL
 * @param nLevels A [3] array of user defined input/output mapping points
 *        (levels) per CHANNEL
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - #NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than
 *         2.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1C3RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_C3R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_C3R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[3];
    const Npp32s* pLevels_native[3];
    int nLevels_native[3];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_C3R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 3)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * 4 channel 8-bit unsigned look-up-table color conversion, not affecting Alpha.
 *
 * The LUT is derived from a set of user defined mapping points through linear
 * interpolation. 
 * Alpha channel is the last channel and is not processed.
 *
 * @param pSrc \ref source_image_pointer.
 * @param nSrcStep \ref source_image_line_step.
 * @param pDst \ref destination_image_pointer.
 * @param nDstStep \ref destination_image_line_step.
 * @param oSizeROI \ref roi_specification.
 * @param pValues Double pointer to an [4] of arrays of user defined OUTPUT
 *        values per CHANNEL
 * @param pLevels Double pointer to an [4] of arrays of user defined INPUT
 *        values per CHANNEL
 * @param nLevels A [4] array of user defined input/output mapping points
 *        (levels) per CHANNEL
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 *        - #NPP_LUT_NUMBER_OF_LEVELS_ERROR if the number of levels is less than
 *         2.
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiLUT_1Linear_18u_1AC4RNative(JNIEnv *env, jclass cls, jobject pSrc, jint nSrcStep, jobject pDst, jint nDstStep, jobject oSizeROI, jobjectArray pValues, jobjectArray pLevels, jintArray nLevels)
{
    // Null-checks for non-primitive arguments
    if (pSrc == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pSrc' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nSrcStep is primitive
    if (pDst == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDst' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nDstStep is primitive
    if (oSizeROI == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSizeROI' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pValues == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pValues' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLevels' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (nLevels == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'nLevels' is null for nppiLUT_Linear_8u_AC4R");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiLUT_Linear_8u_AC4R(pSrc=%p, nSrcStep=%d, pDst=%p, nDstStep=%d, oSizeROI=%p, pValues=%p, pLevels=%p, nLevels=%p)\n",
        pSrc, nSrcStep, pDst, nDstStep, oSizeROI, pValues, pLevels, nLevels);

    // Native variable declarations
    Npp8u* pSrc_native = NULL;
    int nSrcStep_native = 0;
    Npp8u* pDst_native = NULL;
    int nDstStep_native = 0;
    NppiSize oSizeROI_native;
    const Npp32s* pValues_native[4];
    const Npp32s* pLevels_native[4];
    int nLevels_native[4];

    // Obtain native variable values
    PointerData *pSrc_pointerData = initPointerData(env, pSrc);
    if (pSrc_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pSrc_native = (Npp8u*)pSrc_pointerData->getPointer(env);
    nSrcStep_native = (int)nSrcStep;
    PointerData *pDst_pointerData = initPointerData(env, pDst);
    if (pDst_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDst_native = (Npp8u*)pDst_pointerData->getPointer(env);
    nDstStep_native = (int)nDstStep;
    if (!initNative(env, oSizeROI, oSizeROI_native)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pValues, pValues_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, pLevels, pLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!initNative(env, nLevels, nLevels_native, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Native function call
    NppStatus jniResult_native = nppiLUT_Linear_8u_AC4R(pSrc_native, nSrcStep_native, pDst_native, nDstStep_native, oSizeROI_native, pValues_native, pLevels_native, nLevels_native);

    // Write back native variable values
    if (!releasePointerData(env, pSrc_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nSrcStep is primitive
    if (!releasePointerData(env, pDst_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nDstStep is primitive
    // oSizeROI is a read-only structure
    // pValues is an array of native pointers
    // pLevels is an array of native pointers
    if (!releaseNative(env, nLevels_native, nLevels, 4)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates the size of the temporary buffer for graph-cut with 4 neighborhood labeling.
 *
 * @see nppiGraphcutInitAlloc(), nppiGraphcut_32s8u().
 * 
 * @param oSize Graph size.
 * @param pBufSize Pointer to variable that returns the size of the
 *        temporary buffer. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutGetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcutGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiGraphcutGetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutGetSize(oSize=%p, pBufSize=%p)\n",
        oSize, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutGetSize(oSize_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Calculates the size of the temporary buffer for graph-cut with 8 neighborhood labeling.
 *
 * @see nppiGraphcut8InitAlloc(), nppiGraphcut8_32s8u().
 * 
 * @param oSize Graph size.
 * @param pBufSize Pointer to variable that returns the size of the
 *        temporary buffer. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8GetSizeNative(JNIEnv *env, jclass cls, jobject oSize, jintArray pBufSize)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcut8GetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBufSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBufSize' is null for nppiGraphcut8GetSize");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8GetSize(oSize=%p, pBufSize=%p)\n",
        oSize, pBufSize);

    // Native variable declarations
    NppiSize oSize_native;
    int* pBufSize_native;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pBufSize_pointerData = initPointerData(env, pBufSize);
    if (pBufSize_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBufSize_native = (int*)pBufSize_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8GetSize(oSize_native, pBufSize_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, pBufSize_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Initializes graph-cut state structure and allocates additional resources for graph-cut with 8 neighborhood labeling.
 *
 * @see nppiGraphcut_32s8u(), nppiGraphcutGetSize().
 * 
 * @param oSize Graph size
 * @param ppState Pointer to pointer to graph-cut state structure. 
 * @param pDeviceMem pDeviceMem to the sufficient amount of device memory. The CUDA runtime or NPP memory allocators
 *      must be used to allocate this memory. The minimum amount of device memory required
 *      to run graph-cut on a for a specific image size is computed by nppiGraphcutGetSize().
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutInitAllocNative(JNIEnv *env, jclass cls, jobject oSize, jobjectArray ppState, jobject pDeviceMem)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (ppState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppState' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceMem == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceMem' is null for nppiGraphcutInitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutInitAlloc(oSize=%p, ppState=%p, pDeviceMem=%p)\n",
        oSize, ppState, pDeviceMem);

    // Native variable declarations
    NppiSize oSize_native;
    NppiGraphcutState** ppState_native;
    Npp8u* pDeviceMem_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *ppState_pointerData = initPointerData(env, ppState);
    if (ppState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    ppState_native = (NppiGraphcutState**)ppState_pointerData->getPointer(env);
    PointerData *pDeviceMem_pointerData = initPointerData(env, pDeviceMem);
    if (pDeviceMem_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceMem_native = (Npp8u*)pDeviceMem_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutInitAlloc(oSize_native, ppState_native, pDeviceMem_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, ppState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceMem_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Allocates and initializes the graph-cut state structure and additional resources for graph-cut with 8 neighborhood labeling.
 *
 * @see nppiGraphcut8_32s8u(), nppiGraphcut8GetSize().
 * 
 * @param oSize Graph size
 * @param ppState Pointer to pointer to graph-cut state structure. 
 * @param pDeviceMem to the sufficient amount of device memory. The CUDA runtime or NPP memory allocators
 *      must be used to allocate this memory. The minimum amount of device memory required
 *      to run graph-cut on a for a specific image size is computed by nppiGraphcut8GetSize().
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pBufSize
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8InitAllocNative(JNIEnv *env, jclass cls, jobject oSize, jobjectArray ppState, jobject pDeviceMem)
{
    // Null-checks for non-primitive arguments
    if (oSize == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'oSize' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (ppState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'ppState' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pDeviceMem == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pDeviceMem' is null for nppiGraphcut8InitAlloc");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8InitAlloc(oSize=%p, ppState=%p, pDeviceMem=%p)\n",
        oSize, ppState, pDeviceMem);

    // Native variable declarations
    NppiSize oSize_native;
    NppiGraphcutState** ppState_native;
    Npp8u* pDeviceMem_native = NULL;

    // Obtain native variable values
    if (!initNative(env, oSize, oSize_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *ppState_pointerData = initPointerData(env, ppState);
    if (ppState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    ppState_native = (NppiGraphcutState**)ppState_pointerData->getPointer(env);
    PointerData *pDeviceMem_pointerData = initPointerData(env, pDeviceMem);
    if (pDeviceMem_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pDeviceMem_native = (Npp8u*)pDeviceMem_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8InitAlloc(oSize_native, ppState_native, pDeviceMem_native);

    // Write back native variable values
    // oSize is a read-only structure
    if (!releasePointerData(env, ppState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pDeviceMem_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Frees the additional resources of the graph-cut state structure.
 *
 * @see nppiGraphcutInitAlloc
 * @see nppiGraphcut8InitAlloc
 * 
 * @param pState Pointer to graph-cut state structure. 
 *
 * @return
 *  NPP_SUCCESS Indicates no error. Any other value indicates an error
 *         or a warning
 *  NPP_SIZE_ERROR Indicates an error condition if any image dimension
 *         has zero or negative value
 *  NPP_NULL_POINTER_ERROR Indicates an error condition if pState
 *         pointer is NULL
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcutFreeNative(JNIEnv *env, jclass cls, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcutFree");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcutFree(pState=%p)\n",
        pState);

    // Native variable declarations
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pState_pointerData = initPointerData(env, pState);
    if (pState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pState_native = (NppiGraphcutState*)pState_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcutFree(pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Graphcut of a flow network (32bit signed integer edge capacities). The
 * function computes the minimal cut (graphcut) of a 2D regular 4-connected
 * graph. 
 * The inputs are the capacities of the horizontal (in transposed form),
 * vertical and terminal (source and sink) edges. The capacities to source and
 * sink 
 * are stored as capacity differences in the terminals array 
 * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
 * edge capacities 
 * for boundary edges that would connect to nodes outside the specified domain
 * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
 * computed labeling may be wrong!
 * The computed binary labeling is encoded as unsigned 8bit values (0 / 255).
 *
 * @see nppiGraphcutInitAlloc(), nppiGraphcutFree(), nppiGraphcutGetSize().
 *
 * @param pTerminals Pointer to differences of terminal edge capacities
 *        (terminal(x) = source(x) - sink(x))
 * @param pLeftTransposed Pointer to transposed left edge capacities
 *        (left(0,*) must be 0)
 * @param pRightTransposed Pointer to transposed right edge capacities
 *        (right(width-1,*) must be 0)
 * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
 * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
 *        must be 0)
 * @param nStep Step in bytes between any pair of sequential rows of edge
 *        capacities
 * @param nTransposedStep Step in bytes between any pair of sequential
 *        rows of tranposed edge capacities
 * @param size Graph size
 * @param pLabel Pointer to destination label image 
 * @param nLabelStep Step in bytes between any pair of sequential rows of
 *        label image
 * @param pState Pointer to graph-cut state structure. This structure must be
 *          initialized allocated and initialized using nppiGraphcutInitAlloc().
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut_132s8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pBottom, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut_32s8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pBottom=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pBottom, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32s* pTerminals_native = NULL;
    Npp32s* pLeftTransposed_native = NULL;
    Npp32s* pRightTransposed_native = NULL;
    Npp32s* pTop_native = NULL;
    Npp32s* pBottom_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32s*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32s*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32s*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32s*)pTop_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32s*)pBottom_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    PointerData *pState_pointerData = initPointerData(env, pState);
    if (pState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pState_native = (NppiGraphcutState*)pState_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut_32s8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pBottom_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    if (!releasePointerData(env, pState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

/**
 * <pre>
 * Graphcut of a flow network (32bit signed integer edge capacities). The
 * function computes the minimal cut (graphcut) of a 2D regular 8-connected
 * graph. 
 * The inputs are the capacities of the horizontal (in transposed form),
 * vertical and terminal (source and sink) edges. The capacities to source and
 * sink 
 * are stored as capacity differences in the terminals array 
 * ( terminals(x) = source(x) - sink(x) ). The implementation assumes that the
 * edge capacities 
 * for boundary edges that would connect to nodes outside the specified domain
 * are set to 0 (for example left(0,*) == 0). If this is not fulfilled the
 * computed labeling may be wrong!
 * The computed binary labeling is encoded as unsigned 8bit values (0 / 255).
 *
 * @see nppiGraphcut8InitAlloc(), nppiGraphcutFree(), nppiGraphcut8GetSize().
 *
 * @param pTerminals Pointer to differences of terminal edge capacities
 *        (terminal(x) = source(x) - sink(x))
 * @param pLeftTransposed Pointer to transposed left edge capacities
 *        (left(0,*) must be 0)
 * @param pRightTransposed Pointer to transposed right edge capacities
 *        (right(width-1,*) must be 0)
 * @param pTop Pointer to top edge capacities (top(*,0) must be 0)
 * @param pTopLeft Pointer to top left edge capacities (topleft(*,0) 
 *		  & topleft(0,*) must be 0)
 * @param pTopRight Pointer to top right edge capacities (topright(*,0)
 *        & topright(width-1,*) must be 0)
 * @param pBottom Pointer to bottom edge capacities (bottom(*,height-1)
 *        must be 0)
 * @param pBottomLeft Pointer to bottom left edge capacities 
 *        (bottomleft(*,height-1) && bottomleft(0,*) must be 0)
 * @param pBottomRight Pointer to bottom right edge capacities 
 *        (bottomright(*,height-1) && bottomright(width-1,*) must be 0)
 * @param nStep Step in bytes between any pair of sequential rows of edge
 *        capacities
 * @param nTransposedStep Step in bytes between any pair of sequential
 *        rows of tranposed edge capacities
 * @param size Graph size
 * @param pLabel Pointer to destination label image 
 * @param nLabelStep Step in bytes between any pair of sequential rows of
 *        label image
 * @param pState Pointer to graph-cut state structure. This structure must be
 *          initialized allocated and initialized using nppiGraphcut8InitAlloc().
 * @return
 *  \ref image_data_error_codes, \ref roi_error_codes
 * </pre>
 */
JNIEXPORT jint JNICALL Java_jcuda_jnpp_JNppi_nppiGraphcut8_132s8uNative(JNIEnv *env, jclass cls, jobject pTerminals, jobject pLeftTransposed, jobject pRightTransposed, jobject pTop, jobject pTopLeft, jobject pTopRight, jobject pBottom, jobject pBottomLeft, jobject pBottomRight, jint nStep, jint nTransposedStep, jobject size, jobject pLabel, jint nLabelStep, jobject pState)
{
    // Null-checks for non-primitive arguments
    if (pTerminals == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTerminals' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLeftTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLeftTransposed' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pRightTransposed == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pRightTransposed' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTop == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTop' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopLeft' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pTopRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pTopRight' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottom == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottom' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomLeft == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomLeft' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pBottomRight == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pBottomRight' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nStep is primitive
    // nTransposedStep is primitive
    if (size == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'size' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    if (pLabel == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pLabel' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    // nLabelStep is primitive
    if (pState == NULL)
    {
        ThrowByName(env, "java/lang/NullPointerException", "Parameter 'pState' is null for nppiGraphcut8_32s8u");
        return JNPP_STATUS_INTERNAL_ERROR;
    }

    // Log message
    Logger::log(LOG_TRACE, "Executing nppiGraphcut8_32s8u(pTerminals=%p, pLeftTransposed=%p, pRightTransposed=%p, pTop=%p, pTopLeft=%p, pTopRight=%p, pBottom=%p, pBottomLeft=%p, pBottomRight=%p, nStep=%d, nTransposedStep=%d, size=%p, pLabel=%p, nLabelStep=%d, pState=%p)\n",
        pTerminals, pLeftTransposed, pRightTransposed, pTop, pTopLeft, pTopRight, pBottom, pBottomLeft, pBottomRight, nStep, nTransposedStep, size, pLabel, nLabelStep, pState);

    // Native variable declarations
    Npp32s* pTerminals_native = NULL;
    Npp32s* pLeftTransposed_native = NULL;
    Npp32s* pRightTransposed_native = NULL;
    Npp32s* pTop_native = NULL;
    Npp32s* pTopLeft_native = NULL;
    Npp32s* pTopRight_native = NULL;
    Npp32s* pBottom_native = NULL;
    Npp32s* pBottomLeft_native = NULL;
    Npp32s* pBottomRight_native = NULL;
    int nStep_native = 0;
    int nTransposedStep_native = 0;
    NppiSize size_native;
    Npp8u* pLabel_native = NULL;
    int nLabelStep_native = 0;
    NppiGraphcutState* pState_native;

    // Obtain native variable values
    PointerData *pTerminals_pointerData = initPointerData(env, pTerminals);
    if (pTerminals_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTerminals_native = (Npp32s*)pTerminals_pointerData->getPointer(env);
    PointerData *pLeftTransposed_pointerData = initPointerData(env, pLeftTransposed);
    if (pLeftTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLeftTransposed_native = (Npp32s*)pLeftTransposed_pointerData->getPointer(env);
    PointerData *pRightTransposed_pointerData = initPointerData(env, pRightTransposed);
    if (pRightTransposed_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pRightTransposed_native = (Npp32s*)pRightTransposed_pointerData->getPointer(env);
    PointerData *pTop_pointerData = initPointerData(env, pTop);
    if (pTop_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTop_native = (Npp32s*)pTop_pointerData->getPointer(env);
    PointerData *pTopLeft_pointerData = initPointerData(env, pTopLeft);
    if (pTopLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopLeft_native = (Npp32s*)pTopLeft_pointerData->getPointer(env);
    PointerData *pTopRight_pointerData = initPointerData(env, pTopRight);
    if (pTopRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pTopRight_native = (Npp32s*)pTopRight_pointerData->getPointer(env);
    PointerData *pBottom_pointerData = initPointerData(env, pBottom);
    if (pBottom_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottom_native = (Npp32s*)pBottom_pointerData->getPointer(env);
    PointerData *pBottomLeft_pointerData = initPointerData(env, pBottomLeft);
    if (pBottomLeft_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomLeft_native = (Npp32s*)pBottomLeft_pointerData->getPointer(env);
    PointerData *pBottomRight_pointerData = initPointerData(env, pBottomRight);
    if (pBottomRight_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pBottomRight_native = (Npp32s*)pBottomRight_pointerData->getPointer(env);
    nStep_native = (int)nStep;
    nTransposedStep_native = (int)nTransposedStep;
    if (!initNative(env, size, size_native)) return JNPP_STATUS_INTERNAL_ERROR;
    PointerData *pLabel_pointerData = initPointerData(env, pLabel);
    if (pLabel_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pLabel_native = (Npp8u*)pLabel_pointerData->getPointer(env);
    nLabelStep_native = (int)nLabelStep;
    PointerData *pState_pointerData = initPointerData(env, pState);
    if (pState_pointerData == NULL)
    {
        return JNPP_STATUS_INTERNAL_ERROR;
    }
    pState_native = (NppiGraphcutState*)pState_pointerData->getPointer(env);

    // Native function call
    NppStatus jniResult_native = nppiGraphcut8_32s8u(pTerminals_native, pLeftTransposed_native, pRightTransposed_native, pTop_native, pTopLeft_native, pTopRight_native, pBottom_native, pBottomLeft_native, pBottomRight_native, nStep_native, nTransposedStep_native, size_native, pLabel_native, nLabelStep_native, pState_native);

    // Write back native variable values
    if (!releasePointerData(env, pTerminals_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pLeftTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pRightTransposed_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTop_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pTopRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottom_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomLeft_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    if (!releasePointerData(env, pBottomRight_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nStep is primitive
    // nTransposedStep is primitive
    // size is a read-only structure
    if (!releasePointerData(env, pLabel_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;
    // nLabelStep is primitive
    if (!releasePointerData(env, pState_pointerData, 0)) return JNPP_STATUS_INTERNAL_ERROR;

    // Return the result
    jint jniResult;
    jniResult = (jint)jniResult_native;
    return jniResult;
}

